var KeystoreController = Class.create({

    view: null,
    config: null,

    _constructor: function(config) {
        this.view = new KeystoreView();
        this.config = Object.extend(Object.extend({}, KeystoreController.DEFAULT_CONFIG), config);
    },

    start: function() {
        this.view.show({
            onGenerateKeyPair: this._handleGeneratePublicKey.bind(this),
            onGeneratePgpKey: this._handleGeneratePpgKey.bind(this),
            onImportPublicKey: this._handleImportPublicKey.bind(this),
            onImportPgpKey: this._handleImportPgpKey.bind(this)
        });
    },

    _handleGeneratePublicKey: function(view) {
        view.hideMessage();

        var dialog = new GenerateKeyDialog("GenerateSSHKeyDialog", this.config.generateKeyPairUrl);
        dialog.setSize(480, 280);
        dialog.show({
            onDone: this._handleDoneGenerateKey.bind(this)
        });
    },

    _handleGeneratePpgKey: function(view) {
        view.hideMessage();

        var dialog = new GenerateKeyDialog("GeneratePgpKeyDialog", this.config.generatePgpKeyUrl);
        dialog.setSize(480, 326);
        dialog.show({
            onDone: this._handleDoneGenerateKey.bind(this)
        });
    },

    _handleImportPublicKey: function(view) {
        view.hideMessage();

        var dialog = new ImportKeyDialog("ImportSSHKeyDialog", this.config.importPublicKeyUrl);
        dialog.show({
            onDone: this._handleDoneImportKey.bind(this)
        });
    },

    _handleImportPgpKey: function(view) {
        view.hideMessage();

        var dialog = new ImportKeyDialog("ImportPGPKeyDialog", this.config.importPgpKeyUrl);
        dialog.show({
            onDone: this._handleDoneImportKey.bind(this)
        });
    },

    _handleDoneGenerateKey: function(dialog) {
        var message = dialog.getDoneMessage();
        var url = dialog.getDownloadUrl();
        dialog.dispose();

        this.view.showMessage(message);
        this._downloadGeneratedKey(url);
    },

    _handleDoneImportKey: function(dialog) {
        var message = dialog.getDoneMessage();
        dialog.dispose();
        this.view.showMessage(message);
    },

    _downloadGeneratedKey: function(url) {
        var channel = new IFrameChannel();
        var token = "keystore-" + (new Date().getTime()).toString(36);
        channel.setLoadCompleteToken(token);
        channel.open(url, "post");
        channel.send("");
    }
});

KeystoreController.DEFAULT_CONFIG = {
    generateKeyPairUrl: "",
    generatePgpKeyUrl: "",
    importPublicKeyUrl: "",
    importPgpKeyUrl: ""
};