var KeystoreView = Class.create({

    messageBar: null,

    generateSshKeyButton: null,
    generateKeyPairHandler: null,
    importSshKeyButton: null,
    importPublicKeyHandler: null,
    deletePublicKeyButton: null,
    deletePublicKeyHandler: null,

    generatePgpKeyButton: null,
    generatePgpKeyHandler: null,
    importPgpKeyButton: null,
    importPgpKeyHandler: null,
    deletePgpKeyButton: null,
    deletePgpKeyHandler: null,

    listener: null,

    _constructor: function() {
        this.messageBar = $("messagebar");
        this.generateSshKeyButton = $("gensshkey");
        this.generatePgpKeyButton = $("genpgpkey");
        this.importSshKeyButton = $("importsshkey");
        this.importPgpKeyButton = $("importpgpkey");
        this.deletePublicKeyButton = $("delsshpubkey");
        this.deletePgpKeyButton = $("delpgpkey");
    },

    show: function(listener) {
        this.listener = Object.extend(new KeystoreView.Listener(), listener);
        this.bindEvents();
    },

    hide: function() {
        this.unbindEvents();
        this.listener = null;
    },

    showMessage: function(text) {
        this.hideMessage();

        if (text != null) {
            this.messageBar.appendChild(this._createSuccessMessage(text));
        }
    },

    hideMessage: function() {
        while (this.messageBar.hasChildNodes()) {
            this.messageBar.removeChild(this.messageBar.lastChild);
        }
    },

    bindEvents: function() {
        this.generateKeyPairHandler = this._bindHandlerFor(this.generateSshKeyButton, this._handleGenerateKeyPair);
        this.generatePgpKeyHandler = this._bindHandlerFor(this.generatePgpKeyButton, this._handleGeneratePgpKey);
        this.importPublicKeyHandler = this._bindHandlerFor(this.importSshKeyButton, this._handleImportPublicKey);
        this.importPgpKeyHandler = this._bindHandlerFor(this.importPgpKeyButton, this._handleImportPgpKey);
        this.deletePublicKeyHandler = this._handleDeletePublicKey.bindAsEventListener(this);
        this.deletePgpKeyHandler = this._handleDeletePgpKey.bindAsEventListener(this);
    },

    unbindEvents: function() {
        this._unbindHandlerFor(this.generateSshKeyButton, this.generateKeyPairHandler);
        this._unbindHandlerFor(this.generatePgpKeyButton, this.generatePgpKeyHandler);
        this._unbindHandlerFor(this.importSshKeyButton, this.importPublicKeyHandler);
        this._unbindHandlerFor(this.importPgpKeyButton, this.importPgpKeyHandler);
        this.generateKeyPairHandler = null;
        this.generatePgpKeyHandler = null;
        this.importPublicKeyHandler = null;
        this.importPgpKeyHandler = null;
        this.deletePublicKeyHandler = null;
        this.deletePgpKeyHandler = null;
    },

    _bindHandlerFor: function(button, handler) {
        if (button) {
            var f = handler.bindAsEventListener(this);
            Event.observe(button, "click", f);
            return f;
        }
        return null;
    },

    _unbindHandlerFor: function(button, handler) {
        if (button && handler) {
            Event.unobserve(button, "click", handler);
        }
    },

    _handleGenerateKeyPair: function(e) {
        Event.stop(e);
        this.listener.onGenerateKeyPair(this);
    },

    _handleGeneratePgpKey: function(e) {
        Event.stop(e);
        this.listener.onGeneratePgpKey(this);
    },

    _handleImportPublicKey: function(e) {
        Event.stop(e);
        this.listener.onImportPublicKey(this);
    },

    _handleImportPgpKey: function(e) {
        Event.stop(e);
        this.listener.onImportPgpKey(this);
    },

    _handleDeletePublicKey: function(e) {
    },

    _handleDeletePgpKey: function(e) {
    },

    _createSuccessMessage: function(text) {
        var message = document.createElement("H3");
        message.appendChild(document.createTextNode(text));

        var result = document.createElement(Tag.DIV);
        Utils.addCssClass(result, "alertbox");
        result.appendChild(message);

        return result;
    }
});

KeystoreView.Listener = Class.create({
    onGenerateKeyPair: function(view) {},
    onImportPublicKey: function(view) {},
    onDeletePublicKey: function(view) {},
    onGeneratePgpKey: function(view) {},
    onImportPgpKey: function(view) {},
    onDeletePgpKey: function(view) {}
});