var Channel = Class.create({

    timeout: 0,
    preventCaching: false,
    timeoutTimer: null,
    interrupted: false,

    listener: null,

    _constructor: function(config) {
        config = Object.extend(Object.extend({}, Channel.DEFAULT_CONFIG), config);
        this.timeout = config.timeout;
        this.preventCaching = config.preventCaching;
    },

    open: function(listener) {
        this.listener = listener || new Channel.Listener();
        this._createTimeoutTimer();
    },

    send: function() {
        throw "Not implemented."
    },

    abort: function() {
        this.interrupted = true;
        this.close();
    },

    close: function() {
        this._stopTimeoutTimer();
        this.timeoutTimer = null;
    },

    _handleTimeout: function() {
        try {
            this.abort();
        } catch (e) {}

        this.listener.onTimeout(this);
    },

    _createTimeoutTimer: function() {
        if (this.timeout > 0) {
            this.timeoutTimer = new Timer(this._handleTimeout.bind(this), this.timeout);
            this.timeoutTimer.setRepeats(false);
        }
    },

    _startTimeoutTimer: function() {
        if (this.timeoutTimer) {
            this.timeoutTimer.start();
        }
    },

    _stopTimeoutTimer: function() {
        if (this.timeoutTimer) {
            this.timeoutTimer.stop();
        }
    },

    _convertToChannelData: function(data, provider) {
        var result = [];
        for (var key in data) {
            if (!data.hasOwnProperty(key)) {
                continue;
            }
            var value = data[key];
            if (Object.isArray(value)) {
                for (var i = 0; i < value.length; ++i) {
                    result.push(provider(key, value[i]));
                }
            } else {
                result.push(provider(key, value));
            }
        }
        return result;
    }

});

Channel.DEFAULT_CONFIG = {timeout: 0, preventCaching: false};
Channel.Listener = Class.create({
    onComplete: function() {},
    onTimeout: function() {},
    onError: function() {}
});