var ManageItemsPane = Class.create({

    addButton: null,
    deleteButton: null,
    selectAllBox: null,
    selectionBoxes: null,
    confirmDeleteDialog: null,
    isDeleteConfirmed: false,

    _constructor: function(addButton, deleteButton, selectAllBox, itemSelectionBoxes) {
        this.addButton = addButton || $("additem");
        this.deleteButton = deleteButton || $("deleteitem");
        this.selectAllBox = selectAllBox || $("selectall");
        this.selectionBoxes = itemSelectionBoxes || Utils.getFormInputs(this.selectAllBox.form, Tag.INPUT_TYPE_CHECKBOX, "name");
        Utils.setEnabled(this.selectAllBox, this.selectionBoxes.length != 0);

        this._bindEvents();
    },

    getSelectedCount: function() {
        var count = 0;
        for (var i = 0, size = this.selectionBoxes.length; i < size; ++i) {
            if (this.selectionBoxes[i].checked) {
                count++;
            }
        }
        return count;
    },

    deleteConfirmed: function() {
        if (this.confirmDeleteDialog != null) {
            this.confirmDeleteDialog.dispose();
            this.confirmDeleteDialog = null;
        }
        this.isDeleteConfirmed = true;
        this.deleteButton.click();
    },

    _handleDelete: function(event) {
        if (!this.isDeleteConfirmed) {
            if (this.getSelectedCount() != 0) {
                this._displayConfirmDelete();
            }
            Event.stop(event);
        }
    },

    _handleSelectAll: function() {
        var selected = Boolean.toBoolean($F(this.selectAllBox));

        for (var i = 0, size = this.selectionBoxes.length; i < size; ++i) {
            this.selectionBoxes[i].checked = selected;
        }

        this._update();
    },

    _displayConfirmDelete: function() {
        if (this.confirmDeleteDialog == null) {
            this.confirmDeleteDialog = this._createConfirmDeleteDialog();
        }
        this.isDeleteConfirmed = false;
        this.confirmDeleteDialog.setVisible(true);
    },

    _createConfirmDeleteDialog: function() {
        var dialog = new ModalDialog("DeleteItemsConfirmDialog");
        var content = dialog.getContent();

        var messagePanel = document.createElement(Tag.PARAGRAPH);
        Utils.setStyle(messagePanel, "font-weight:bold;");
        messagePanel.appendChild(document.createTextNode(ManageItemsPane.Messages.CONFIRM_DELETE));
        content.appendChild(messagePanel);

        dialog.addButton(ModalDialog.BUTTON_OK, ManageItemsPane.Buttons.OK, this.deleteConfirmed.bind(this));
        dialog.addButton(ModalDialog.BUTTON_CANCEL, ManageItemsPane.Buttons.CANCEL);

        return dialog;
    },

    _update: function() {
        var count = this.getSelectedCount();
        this.selectAllBox.checked = (this.selectionBoxes.length == count);
        Utils.setEnabled(this.deleteButton, count != 0);
    },

    _bindEvents: function () {
        Event.observe(this.selectAllBox, "click", this._handleSelectAll.bind(this));
        Event.observe(this.deleteButton, "click", this._handleDelete.bindAsEventListener(this));
        var selectionHandler = this._update.bind(this);

        for (var i = 0; i < this.selectionBoxes.length; ++i) {
            Event.observe(this.selectionBoxes[i], "click", selectionHandler);
        }
    }

});

ManageItemsPane.Buttons = {OK: "OK", CANCEL: "Cancel"};
ManageItemsPane.Messages = {CONFIRM_DELETE: "Delete selected item(s)?"};
