var UploadFormDialog = Class.create(ModalDialog, {

    formsBox: null,
    formDescElement: null,

    _constructor: function(forms) {
        this.formsBox = document.createElement("SELECT");
        this.formsBox.name = "formname";
        this.formsBox.id = this.formsBox.name;
        this.formsBox.onchange = this._update.bind(this);

        this.formDescElement = document.createElement(Tag.SPAN);
        this.formDescElement.id = "formdescription";

        this.setContent(this._createContent(forms));

        this.addButton(ModalDialog.BUTTON_OK, UploadFormDialog.Buttons.OK);
        this.addButton(ModalDialog.BUTTON_CANCEL, UploadFormDialog.Buttons.CANCEL);
        this.addButton(ModalDialog.BUTTON_CANCEL, UploadFormDialog.Buttons.CANCEL_UPLOAD, this.onCancelUpload.bind(this));
        this.setTitle(UploadForm.Messages.TITLE);
    },

    getId: function() {
        return "UploadFormSelectionDialog";
    },

    show: function(listener) {
        var viewport = document.viewport.getDimensions();
        var width = Math.max(400, viewport.width * 0.4);
        var height = Math.max(245, viewport.height * 0.37);
        this._super.setSize.call(this, width, height);

        this._update();

        this._super.show.call(this, Object.extend(new UploadFormDialog.Listener(), listener || {}));
    },

    hide: function() {
        this._super.hide.call(this);
        this.dispose();
    },

    onSubmit: function() {
        this.listener.onSubmit(this, $F(this.formsBox));
    },

    onCancel: function() {
        this.listener.onCancel(this);
    },

    onCancelUpload: function() {
        this.listener.onCancelUpload(this);
    },

    _createContent: function(forms) {
        for (var i = 0; i < forms.length; ++i) {
            var form = forms[i];

            var item = document.createElement(Tag.OPTION);
            item.appendChild(document.createTextNode(form.name));
            item.value = form.name;
            item.title = form.description;
            this.formsBox.appendChild(item);
        }

        var panel = document.createElement(Tag.DIV);

        var infoPanel = document.createElement(Tag.DIV);
        infoPanel.appendChild(document.createTextNode(UploadFormDialog.Messages.INFO));
        Utils.setStyle(infoPanel, "padding: 2em 1.5em 2em 4em;");
        panel.appendChild(infoPanel);

        var formPanel = document.createElement(Tag.DIV);
        formPanel.appendChild(Utils.createLabel(UploadFormDialog.Labels.FORM, this.formsBox.id));
        formPanel.appendChild(this.formsBox);
        formPanel.appendChild(document.createElement("BR"));
        formPanel.appendChild(Utils.createLabel(UploadFormDialog.Labels.DESCRIPTION, this.formDescElement.id));
        formPanel.appendChild(this.formDescElement);
        Utils.setStyle(formPanel, "padding-left: 5em; line-height: 2");
        panel.appendChild(formPanel);

        return panel;
    },

    _update: function() {
        if (Object.isElement(this.formsBox)) {
            var description = this._descriptionFor($F(this.formsBox));
            this._setDescription(description || "");
        }
    },

    _descriptionFor: function(name) {
        for (var i = 0; i < this.formsBox.options.length; ++i) {
            var item = this.formsBox.options[i];
            if (name == item.value) {
                return item.title;
            }
        }
        return null;
    },

    _setDescription: function(description) {
        var descElement = document.createTextNode(description);

        if (this.formDescElement.firstChild) {
            this.formDescElement.replaceChild(descElement, this.formDescElement.firstChild);
        } else {
            this.formDescElement.appendChild(descElement);
        }
    }

});
UploadFormDialog.Buttons = {OK: "OK", CANCEL: "Cancel", CANCEL_UPLOAD: "Cancel Upload"};
UploadFormDialog.Labels = {FORM: "Form", DESCRIPTION: "Description"};
UploadFormDialog.Messages = {INFO: "Select form to use for file upload and click OK or click Cancel to upload files without form data."};
UploadFormDialog.Listener = Class.create(ModalDialog.Listener, {
    onCancelUpload: function(d) {}
});