jscape.DOMAIN_TEMPLATE = {
    "name": "",
    "description": "",
    "serviceDescriptors": []
};

jscape.DEFAULT_SERVICES = [
        {
            "@class": "com.jscape.inet.mft.management.descriptors.StatisticsServiceDescriptor"
        },
        {
            "@class": "com.jscape.inet.mft.management.descriptors.AccountTemplateServiceDescriptor"
        },
        {
            "@class": "com.jscape.inet.mft.management.descriptors.AccountAuthenticationServiceDescriptor"
        },
        {
            "@class": "com.jscape.inet.mft.management.descriptors.EventServiceDescriptor"
        },
        {
            "@class": "com.jscape.inet.mft.management.descriptors.ReportServiceDescriptor"
        },
        {
            "@class": "com.jscape.inet.mft.management.descriptors.ResourceServiceDescriptor"
        },
        {
            "@class": "com.jscape.inet.mft.management.descriptors.IPAccessServiceDescriptor"
        },
        {
            "@class": "com.jscape.inet.mft.management.descriptors.PasswordServiceDescriptor"
        },
        {
            "@class": "com.jscape.inet.mft.management.descriptors.DirectoryMonitorServiceDescriptor"
        },
        {
            "@class": "com.jscape.inet.mft.DropZoneServiceDescriptor"
        },
        {
            "@class": "com.jscape.inet.mft.WebThemeServiceDescriptor"
        },
        {
            "@class": "com.jscape.inet.mft.TradingPartnerServiceDescriptor"
        },
        {
            "@class": "com.jscape.inet.mft.AddressBookServiceDescriptor"
        }
];


jscape.addressesOf = function(interfaces) {
    var addresses = ["0.0.0.0"];

    for (var i = 0; i < interfaces.length; ++i) {
        var networkInterface = interfaces[i];
        for (var j = 0; j < networkInterface.addresses.length; ++j) {
            addresses.push(networkInterface.addresses[j]);
        }
    }

    return addresses;
};

jscape.aliasesOf = function(keys) {
    var aliases = [];
    for (var i = 0; i < keys.length; ++i) {
        aliases.push(keys[i].alias);
    }

    return aliases;
};


jscape.descriptorAllowed = function(descriptor, serverParameters) {
    var descriptorClass = descriptor["@class"];

    for (var i = 0; i < serverParameters.allowedDescriptors.length; ++i) {
        if (serverParameters.allowedDescriptors[i] === descriptorClass) {
            return true;
        }
    }

    return false;
};


jscape.BadDomainNameException = function(){};

jscape.BadDirectoryException = function(){};

jscape.BadUrlException = function(){};

jscape.BadUsernameException = function(){};

jscape.DomainExistsException = function(){};

jscape.ServiceControllerException = function(){};

jscape.ServiceDeniedException = function(descriptor) {
    this.descriptor = descriptor;
};


jscape.DomainNameController = function(dialog, panel) {
    this.dialog = dialog;
    this.panel = panel;

    this.listener = null;
};

jscape.DomainNameController.prototype = {

    start: function(listener, serverParameters, serverKeys) {
        this.listener = listener;

        this.dialog.select(this.panel);
    },

    reset: function() {
        this.panel.reset();
    },

    validate: function() {
        this.panel.clearErrors();

        try {
            this._validatePanel();
            this.listener.onValidated();
        } catch (e) {
            this._processInputError(e);
        }
    },

    onErrorShown: function() {
        this.listener.onValidationError();
    },

    getDomainName: function() {
        return this.panel.getDomainName();
    },

    getDomainDescription: function() {
        return this.panel.getDomainDescription();
    },

    _validatePanel: function() {
        if (this.panel.getDomainName().length === 0) {
            throw new jscape.BadDomainNameException();
        }
    },

    _processInputError: function(error) {
        if (error instanceof jscape.BadDomainNameException) {
            this.panel.showBadNameError(this);
        }
    }
};

jscape.DomainNamePanel = function() {
    this.panel = $("#domainNamePanel");
    this.nameField = new jscape.TextField($("#domainNamePanelNameField"));
    this.descriptionField = new jscape.TextField($("#domainNamePanelDescriptionField"));
    this.nameErrorLabel = $("#domainNamePanelNameErrorLabel");

    this.alertFadeTimer = this._createAlertTimer();
};

jscape.DomainNamePanel.prototype = {

    show: function() {
        this.panel.show();
    },

    hide: function() {
        this.panel.hide();
    },

    getDomainName: function() {
        return this.nameField.getValue();
    },

    setDomainName: function(value) {
        this.nameField.setValue(value);
    },

    getDomainDescription: function() {
        return this.descriptionField.getValue();
    },

    setDomainDescription: function(value) {
        this.descriptionField.setValue(value);
    },

    reset: function() {
        this.setDomainName("");
        this.setDomainDescription("");
    },

    clearErrors: function() {
        this.alertFadeTimer.cancel();
        this.nameErrorLabel.hide();
    },

    showBadNameError: function() {
        this.nameErrorLabel.show();
        this.alertFadeTimer.start();
    },

    _createAlertTimer: function() {
        var view = this;
        return new jscape.Timer(function(){
            view.clearErrors();
        }, 5000, false);
    }
};


jscape.DomainServiceController = function(dialog, panel, controllers) {
    this.dialog = dialog;
    this.panel = panel;
    this.controllers = controllers;

    this.listener = null;
    this.serverParameters = null;
    this.serverKeys = null;
};

jscape.DomainServiceController.prototype = {

    start: function(listener, serverParameters, serverKeys) {
        this.listener = listener;
        this.serverParameters = serverParameters;
        this.serverKeys = serverKeys;

        this._startControllers();

        this.dialog.select(this.panel);
    },

    reset: function() {
        this.panel.reset();
    },

    validate: function() {
        this.panel.apply();
    },

    onControllerCompleted: function(descriptor) {
        this.descriptor = descriptor;
        this.listener.onValidated(this);
    },

    onControllerBroken: function() {
        this.listener.onValidationError(this);
    },

    getDescriptor: function() {
        return this.descriptor;
    },

    _startControllers: function() {
        for (var i = 0; i < this.controllers.length; ++i) {
            this.controllers[i].start(this,
                                      this.serverParameters,
                                      this.serverKeys);
        }
    }
};


jscape.DomainServicePanel = function(panelId, serviceBoxId, panels) {
    this.panel = $("#" + panelId);
    this.serviceBox = new jscape.ComboBox($("#" + serviceBoxId));
    this.panels = panels;

    this._init();
};


jscape.DomainServicePanel.prototype = {

    show: function() {
        this.panel.show();
        this._onServiceBoxChange();
    },

    hide: function() {
        this.panel.hide();
    },

    reset: function() {
        this.serviceBox.setValue(0);
    },

    apply: function() {
        var index = this.serviceBox.getValue();
        var panel = this.panels[index];
        panel.applyChanges();
    },

    _init: function() {
        this._initListeners();
    },

    _initListeners: function() {
        var view = this;

        this.serviceBox.field.change(function(){
            view._onServiceBoxChange();
        });
    },

    _onServiceBoxChange: function() {
        for (var i = 0; i < this.panels.length; ++i) {
            this.panels[i].hide();
        }

        var index = this.serviceBox.getValue();
        this.panels[index].show();
    }
};



jscape.FtpServiceController = function(panel) {
    this.panel = panel;

    this.listener = null;
};

jscape.FtpServiceController.prototype = {

    DESCRIPTOR_CLASS: "com.jscape.inet.mft.management.descriptors.FtpServiceDescriptor",
    DEFAULT_PORT: 21,

    start: function(listener, parameters, keys) {
        this.listener = listener;

        var addresses = jscape.addressesOf(parameters.interfaces);
        this.panel.setAddresses(addresses);

        this.panel.setPort(this.DEFAULT_PORT);

        var aliases = jscape.aliasesOf(keys);
        this.panel.setKeys(aliases);

        this.panel.reset();

        this.panel.setListener(this);
    },

    onApply: function() {
        var descriptor = this.getDescriptor();
        this.listener.onControllerCompleted(descriptor);
    },

    getDescriptor: function() {
        return {
            "@class": this.DESCRIPTOR_CLASS,
            host: this.panel.getHost(),
            port: this.panel.getPort(),
            keyAlias: this.panel.getKey(),
            type: this.panel.getType()
        };
    },

    setDescriptor: function(descriptor) {
        this.panel.setHost(descriptor.host);
        this.panel.setPort(descriptor.port);
        this.panel.setKey(descriptor.keyAlias);
        this.panel.setType(descriptor.type);
    }

};


jscape.FtpServicePanel = function() {
    this.panel = $("#ftpServicePanel");
    this.hostBox = new jscape.ComboBox($("#ftpServicePanelHostBox"));
    this.portField = $("#ftpServicePanelPortField");
    this.typeBox = new jscape.ComboBox($("#ftpServicePanelTypeBox"));
    this.keysBox = new jscape.ComboBox($("#ftpServicePanelKeyBox"));

    this.listener = null;

    this._init();
};

jscape.FtpServicePanel.prototype = {

    ADDRESS_TEMPLATE: "<option value='{address}'>{address}</option>",
    KEY_TEMPLATE: "<option value='{alias}'>{alias}</option>",

    show: function() {
        this.panel.show();
    },

    hide: function() {
        this.panel.hide();
    },

    reset: function() {
        // todo
    },

    setListener: function(listener) {
        this.listener = listener;
    },

    applyChanges: function() {
        this.listener.onApply();
    },

    setAddresses: function(addresses) {
        this.hostBox.field.html("");
        for (var i = 0; i < addresses.length; ++i) {
            var address = addresses[i];
            this.hostBox.field.append(this.ADDRESS_TEMPLATE.supplant({address: address}));
        }
    },

    getHost: function() {
        return this.hostBox.getValue();
    },

    setHost: function(value) {
        this.hostBox.setValue(value);
    },

    getPort: function() {
        return this.portField.spinner("value");
    },

    setPort: function(value) {
        this.portField.spinner("value", value);
    },

    getKey: function() {
        return this.keysBox.getValue();
    },

    setKey: function(value) {
        this.keysBox.setValue(value);
    },

    setKeys: function(keys) {
        this.keysBox.field.html("");
        for (var i = 0; i < keys.length; ++i) {
            var alias = keys[i];
            this.keysBox.field.append(this.KEY_TEMPLATE.supplant({alias: alias}));
        }
    },

    getType: function() {
        return this.typeBox.getValue();
    },

    setType: function(value) {
        this.typeBox.setValue(value);
    },

    _init: function() {
        this.portField.spinner({min: 1, max: 65535});
    }
};



jscape.SftpServiceController = function(panel) {
    this.panel = panel;

    this.listener = null;
};

jscape.SftpServiceController.prototype = {

    DESCRIPTOR_CLASS: "com.jscape.inet.mft.management.descriptors.SftpServiceDescriptor",
    DEFAULT_PORT: 22,

    start: function(listener, parameters, keys) {
        this.listener = listener;

        var addresses = jscape.addressesOf(parameters.interfaces);
        this.panel.setAddresses(addresses);

        this.panel.setPort(this.DEFAULT_PORT);

        var aliases = jscape.aliasesOf(keys);
        this.panel.setKeys(aliases);

        this.panel.reset();

        this.panel.setListener(this);
    },

    onApply: function() {
        var descriptor = this.getDescriptor();
        this.listener.onControllerCompleted(descriptor);
    },

    getDescriptor: function() {
        return {
            "@class": this.DESCRIPTOR_CLASS,
            host: this.panel.getHost(),
            port: this.panel.getPort(),
            keyAlias: this.panel.getKey(),
            authenticationType: this.panel.getAuthenticationType()
        };
    },    

    setDescriptor: function(descriptor) {
        this.panel.setHost(descriptor.host);
        this.panel.setPort(descriptor.port);
        this.panel.setKey(descriptor.keyAlias);
        this.panel.setAuthenticationType(descriptor.authenticationType);
    }

};

jscape.SftpServicePanel = function() {
    this.panel = $("#sftpServicePanel");
    this.hostBox = new jscape.ComboBox($("#sftpServicePanelHostBox"));
    this.portField = $("#sftpServicePanelPortField");
    this.keysBox = new jscape.ComboBox($("#sftpServicePanelKeyBox"));
    this.authenticationBox = new jscape.ComboBox($("#sftpServicePanelAuthenticationBox"));

    this.listener = null;

    this._init();
};

jscape.SftpServicePanel.prototype = {

    ADDRESS_TEMPLATE: "<option value='{address}'>{address}</option>",
    KEY_TEMPLATE: "<option value='{alias}'>{alias}</option>",

    show: function() {
        this.panel.show();
    },

    hide: function() {
        this.panel.hide();
    },

    reset: function() {
        // todo
    },

    setListener: function(listener) {
        this.listener = listener;
    },

    applyChanges: function() {
        this.listener.onApply();
    },

    setAddresses: function(addresses) {
        this.hostBox.field.html("");
        for (var i = 0; i < addresses.length; ++i) {
            var address = addresses[i];
            this.hostBox.field.append(this.ADDRESS_TEMPLATE.supplant({address: address}));
        }
    },

    getHost: function() {
        return this.hostBox.getValue();
    },

    setHost: function(value) {
        this.hostBox.setValue(value);
    },

    getPort: function() {
        return this.portField.spinner("value");
    },

    setPort: function(value) {
        this.portField.spinner("value", value);
    },

    getKey: function() {
        return this.keysBox.getValue();
    },

    setKey: function(value) {
        this.keysBox.setValue(value);
    },

    setKeys: function(keys) {
        this.keysBox.field.html("");
        for (var i = 0; i < keys.length; ++i) {
            var alias = keys[i];
            this.keysBox.field.append(this.KEY_TEMPLATE.supplant({alias: alias}));
        }
    },

    getAuthenticationType: function() {
        return this.authenticationBox.getValue();
    },

    setAuthenticationType: function(value) {
        this.authenticationBox.setValue(value);
    },

    _init: function() {
        this.portField.spinner({min: 1, max: 65535});
    }
};


jscape.AftpServiceController = function(panel) {
    this.panel = panel;

    this.listener = null;
};

jscape.AftpServiceController.prototype = {

    DESCRIPTOR_CLASS: "com.jscape.inet.mft.AftpServiceDescriptor",
    DEFAULT_PORT: 3000,

    start: function(listener, parameters, keys) {
        this.listener = listener;

        var addresses = jscape.addressesOf(parameters.interfaces);
        this.panel.setAddresses(addresses);

        this.panel.setPort(this.DEFAULT_PORT);

        var aliases = jscape.aliasesOf(keys);
        this.panel.setKeys(aliases);

        this.panel.reset();

        this.panel.setListener(this);
    },

    onApply: function() {
        var descriptor = this.getDescriptor();
        this.listener.onControllerCompleted(descriptor);
    },

    getDescriptor: function() {
        return {
            "@class": this.DESCRIPTOR_CLASS,
            address: {
                host: this.panel.getHost(),
                port: this.panel.getPort()
            },
            keyAlias: this.panel.getKey(),
            protectionRequired: this.panel.isProtectionRequired()

        };
    },

    setDescriptor: function(descriptor) {
        this.panel.setHost(descriptor.address.host);
        this.panel.setPort(descriptor.address.port);
        this.panel.setKey(descriptor.keyAlias);
        this.panel.setProtectionRequired(descriptor.protectionRequired);
    }

};

jscape.AftpServicePanel = function() {
    this.panel = $("#aftpServicePanel");
    this.hostBox = new jscape.ComboBox($("#aftpServicePanelHostBox"));
    this.portField = $("#aftpServicePanelPortField");
    this.keysBox = new jscape.ComboBox($("#aftpServicePanelKeyBox"));
    this.protectionRequiredBox = new jscape.CheckBox($("#aftpServicePanelSecuredConnectionRequiredBox"));

    this.listener = null;

    this._init();
};

jscape.AftpServicePanel.prototype = {

    ADDRESS_TEMPLATE: "<option value='{address}'>{address}</option>",
    KEY_TEMPLATE: "<option value='{alias}'>{alias}</option>",

    show: function() {
        this.panel.show();
    },

    hide: function() {
        this.panel.hide();
    },

    reset: function() {
        // todo
    },

    setListener: function(listener) {
        this.listener = listener;
    },

    applyChanges: function() {
        this.listener.onApply();
    },

    setAddresses: function(addresses) {
        this.hostBox.field.html("");
        for (var i = 0; i < addresses.length; ++i) {
            var address = addresses[i];
            this.hostBox.field.append(this.ADDRESS_TEMPLATE.supplant({address: address}));
        }
    },

    getHost: function() {
        return this.hostBox.getValue();
    },

    setHost: function(value) {
        this.hostBox.setValue(value);
    },

    getPort: function() {
        return this.portField.spinner("value");
    },

    setPort: function(value) {
        this.portField.spinner("value", value);
    },

    getKey: function() {
        return this.keysBox.getValue();
    },

    setKey: function(value) {
        this.keysBox.setValue(value);
    },

    setKeys: function(keys) {
        this.keysBox.field.html("");
        for (var i = 0; i < keys.length; ++i) {
            var alias = keys[i];
            this.keysBox.field.append(this.KEY_TEMPLATE.supplant({alias: alias}));
        }
    },

    isProtectionRequired: function() {
        return this.protectionRequiredBox.isSelected();
    },

    setProtectionRequired: function(value) {
        this.protectionRequiredBox.setSelected(value);
    },

    _init: function() {
        this.portField.spinner({min: 1, max: 65535});
    }
};


jscape.HttpServiceController = function(panel) {
    this.panel = panel;

    this.listener = null;
};

jscape.HttpServiceController.prototype = {

    DESCRIPTOR_CLASS: "com.jscape.inet.mft.management.descriptors.HttpServiceDescriptor",

    start: function(listener, parameters, keys) {
        this.listener = listener;

        this.panel.reset();

        this.panel.setListener(this);
    },

    onApply: function() {
        var descriptor = this.getDescriptor();
        this.listener.onControllerCompleted(descriptor);
    },

    getDescriptor: function() {
        return {
            "@class": this.DESCRIPTOR_CLASS,
            httpAllowed: this.panel.isHttpEnabled(),
            httpsAllowed: this.panel.isHttpsEnabled()
        };
    },

    setDescriptor: function(descriptor) {
        this.panel.setHttpEnabled(descriptor.httpAllowed);
        this.panel.setHttpsEnabled(descriptor.httpsAllowed);
    }

};

jscape.HttpServicePanel = function() {
    this.panel = $("#httpServicePanel");
    this.httpEnabledBox = new jscape.CheckBox($("#httpServicePanelHttpEnabledBox"));
    this.httpsEnabledBox = new jscape.CheckBox($("#httpServicePanelHttpsEnabledBox"));

    this.listener = null;
};

jscape.HttpServicePanel.prototype = {

    show: function() {
        this.panel.show();
    },

    hide: function() {
        this.panel.hide();
    },

    reset: function() {
        this.setHttpEnabled(true);
        this.setHttpsEnabled(true);
    },

    setListener: function(listener) {
        this.listener = listener;
    },

    applyChanges: function() {
        this.listener.onApply();
    },    

    isHttpEnabled: function() {
        return this.httpEnabledBox.isSelected();
    },

    setHttpEnabled: function(value) {
        this.httpEnabledBox.setSelected(value);
    },

    isHttpsEnabled: function() {
        return this.httpsEnabledBox.isSelected();
    },

    setHttpsEnabled: function(value) {
        this.httpsEnabledBox.setSelected(value);
    }
};


jscape.WebDavServiceController = function(panel) {
    this.panel = panel;

    this.listener = null;
};

jscape.WebDavServiceController.prototype = {

    DESCRIPTOR_CLASS: "com.jscape.inet.mft.management.descriptors.WebDavServiceDescriptor",

    start: function(listener, parameters, keys) {
        this.listener = listener;

        this.panel.reset();

        this.panel.setListener(this);
    },

    onApply: function() {
        var descriptor = this.getDescriptor();
        this.listener.onControllerCompleted(descriptor);
    },

    getDescriptor: function() {
        return {
            "@class": this.DESCRIPTOR_CLASS,
            httpAllowed: this.panel.isHttpEnabled(),
            httpsAllowed: this.panel.isHttpsEnabled()
        };
    },

    setDescriptor: function(descriptor) {
        this.panel.setHttpEnabled(descriptor.httpAllowed);
        this.panel.setHttpsEnabled(descriptor.httpsAllowed);
    }

};


jscape.WebDavServicePanel = function() {
    this.panel = $("#webDavServicePanel");
    this.httpEnabledBox = new jscape.CheckBox($("#webDavServicePanelHttpEnabledBox"));
    this.httpsEnabledBox = new jscape.CheckBox($("#webDavServicePanelHttpsEnabledBox"));

    this.listener = null;
};

jscape.WebDavServicePanel.prototype = {

    show: function() {
        this.panel.show();
    },

    hide: function() {
        this.panel.hide();
    },

    reset: function() {
        this.setHttpEnabled(true);
        this.setHttpsEnabled(true);
    },

    setListener: function(listener) {
        this.listener = listener;
    },

    applyChanges: function() {
        this.listener.onApply();
    },

    isHttpEnabled: function() {
        return this.httpEnabledBox.isSelected();
    },

    setHttpEnabled: function(value) {
        this.httpEnabledBox.setSelected(value);
    },

    isHttpsEnabled: function() {
        return this.httpsEnabledBox.isSelected();
    },

    setHttpsEnabled: function(value) {
        this.httpsEnabledBox.setSelected(value);
    }
};


jscape.FileRecordDatastoreServiceController = function(panel) {
    this.panel = panel;

    this.listener = null;
};

jscape.FileRecordDatastoreServiceController.prototype = {

    DESCRIPTOR_CLASS: "com.jscape.inet.mft.FileRecordStoreServiceDescriptor",
    DEFAULT_DIRECTORY: "%installdir%/users/%domain%",

    start: function(listener, parameters, keys) {
        this.listener = listener;

        this.panel.reset();

        this.panel.setDirectory(this.DEFAULT_DIRECTORY);

        this.panel.setListener(this);
    },

    onApply: function() {
        try {
            this._validate();
            var descriptor = this.getDescriptor();
            this.listener.onControllerCompleted(descriptor);
        } catch (e) {
            this._processInputError(e);
            this.listener.onControllerBroken();
        }
    },

    getDescriptor: function() {
        return {
            "@class": this.DESCRIPTOR_CLASS,
            directory: this.panel.getDirectory()
        };
    },

    setDescriptor: function(descriptor) {
        this.panel.setDirectory(descriptor.directory);
    },

    _validate: function() {
        this.panel.clearErrors();

        if (!this.panel.getDirectory().length) {
            throw new jscape.BadDirectoryException();
        }
    },

    _processInputError: function(error) {
        this.panel.showBadDirectoryError();
    }

};


jscape.FileRecordDatastoreServicePanel = function() {
    this.panel = $("#fileRecordDatastoreServicePanel");
    this.directoryField = new jscape.TextField($("#fileRecordDatastoreServicePanelDirectoryField"));
    this.directoryErrorLabel = $("#fileRecordDatastoreServicePanelDirectoryErrorLabel");

    this.alertFadeTimer = this._createAlertTimer();

    this.listener = null;
};

jscape.FileRecordDatastoreServicePanel.prototype = {

    show: function() {
        this.panel.show();
    },

    hide: function() {
        this.panel.hide();
    },

    reset: function() {
        this.setDirectory("");
    },

    setListener: function(listener) {
        this.listener = listener;
    },

    applyChanges: function() {
        this.listener.onApply();
    },

    getDirectory: function() {
        return this.directoryField.getValue();
    },

    setDirectory: function(value) {
        this.directoryField.setValue(value);
    },

    showBadDirectoryError: function() {
        this.directoryErrorLabel.show();
        this.alertFadeTimer.start();
    },

    clearErrors: function() {
        this.alertFadeTimer.cancel();
        this.directoryErrorLabel.hide();
    },

    _createAlertTimer: function() {
        var view = this;
        return new jscape.Timer(function(){
            view.clearErrors();
        }, 5000, false);
    }
};


jscape.FileDatastoreServiceController = function(panel) {
    this.panel = panel;

    this.listener = null;
};

jscape.FileDatastoreServiceController.prototype = {

    DESCRIPTOR_CLASS: "com.jscape.inet.mft.management.descriptors.FileStoreServiceDescriptor",
    DEFAULT_DIRECTORY: "%installdir%/users/%domain%",

    start: function(listener, parameters, keys) {
        this.listener = listener;

        this.panel.reset();

        this.panel.setDirectory(this.DEFAULT_DIRECTORY);

        this.panel.setListener(this);
    },

    onApply: function() {
        try {
            this._validate();
            var descriptor = this.getDescriptor();
            this.listener.onControllerCompleted(descriptor);
        } catch (e) {
            this._processInputError(e);
            this.listener.onControllerBroken();
        }
    },

    getDescriptor: function() {
        return {
            "@class": this.DESCRIPTOR_CLASS,
            directory: this.panel.getDirectory()
        };
    },

    setDescriptor: function(descriptor) {
        this.panel.setDirectory(descriptor.directory);
    },

    _validate: function() {
        this.panel.clearErrors();

        if (!this.panel.getDirectory().length) {
            throw new jscape.BadDirectoryException();
        }
    },

    _processInputError: function(error) {
        this.panel.showBadDirectoryError();
    }

};


jscape.FileDatastoreServicePanel = function() {
    this.panel = $("#fileDatastoreServicePanel");
    this.directoryField = new jscape.TextField($("#fileDatastoreServicePanelDirectoryField"));
    this.directoryErrorLabel = $("#fileDatastoreServicePanelDirectoryErrorLabel");
    
    this.alertFadeTimer = this._createAlertTimer();

    this.listener = null;
};

jscape.FileDatastoreServicePanel.prototype = {

    show: function() {
        this.panel.show();
    },

    hide: function() {
        this.panel.hide();
    },

    reset: function() {
        this.setDirectory("");
    },

    setListener: function(listener) {
        this.listener = listener;
    },

    applyChanges: function() {
        this.listener.onApply();
    },

    getDirectory: function() {
        return this.directoryField.getValue();
    },

    setDirectory: function(value) {
        this.directoryField.setValue(value);
    },

    showBadDirectoryError: function() {
        this.directoryErrorLabel.show();
        this.alertFadeTimer.start();
    },

    clearErrors: function() {
        this.alertFadeTimer.cancel();
        this.directoryErrorLabel.hide();
    },

    _createAlertTimer: function() {
        var view = this;
        return new jscape.Timer(function(){
            view.clearErrors();
        }, 5000, false);
    }
};


jscape.DbDatastoreServiceController = function(panel) {
    this.panel = panel;

    this.listener = null;
};

jscape.DbDatastoreServiceController.prototype = {

    DESCRIPTOR_CLASS: "com.jscape.inet.mft.management.descriptors.DbStoreServiceDescriptor",
    DEFAULT_POOL: 10,
    DEFAULT_TTL: 10 * 60 * 1000,

    start: function(listener, parameters, keys) {
        this.listener = listener;

        this.panel.reset();

        this.panel.setPool(this.DEFAULT_POOL);
        this.panel.setTtl(this.DEFAULT_TTL);

        this.panel.setListener(this);
    },

    onApply: function() {
        try {
            this._validate();
            var descriptor = this.getDescriptor();
            this.listener.onControllerCompleted(descriptor);
        } catch (e) {
            this._processInputError(e);
            this.listener.onControllerBroken();
        }
    },

    getDescriptor: function() {
        return {
            "@class": this.DESCRIPTOR_CLASS,
            url: this.panel.getUrl(),
            username: this.panel.getUsername(),
            password: this.panel.getPassword(),
            poolSize: this.panel.getPool(),
            idleConnectionTTL: this.panel.getTtl()
        };
    },

    setDescriptor: function(descriptor) {
        this.panel.setUrl(descriptor.url);
        this.panel.setUsername(descriptor.username);
        this.panel.setPassword(descriptor.password);
        this.panel.setPool(descriptor.poolSize);
        this.panel.setTtl(descriptor.idleConnectionTTL);
    },

    _validate: function() {
        this.panel.clearErrors();

        if (!this.panel.getUrl().length) {
            throw new jscape.BadUrlException();
        }

        if (!this.panel.getUsername().length) {
            throw new jscape.BadUsernameException();
        }
    },

    _processInputError: function(error) {
        if (error instanceof jscape.BadUrlException) {
            this.panel.showBadUrlError();
        } else if (error instanceof jscape.BadUsernameException) {
            this.panel.showBadUsernameError();
        }
    }

};

jscape.DbDatastoreServicePanel = function() {
    this.panel = $("#dbDatastoreServicePanel");
    this.urlField = new jscape.TextField($("#dbDatastoreServicePanelUrlField"));
    this.usernameField = new jscape.TextField($("#dbDatastoreServicePanelUsernameField"));
    this.passwordField = new jscape.TextField($("#dbDatastoreServicePanelPasswordField"));
    this.poolField = $("#dbDatastoreServicePanelPoolField");
    this.ttlField = $("#dbDatastoreServicePanelTtlField");
    this.urlErrorLabel = $("#dbDatastoreServicePanelUrlErrorLabel");
    this.usernameErrorLabel = $("#dbDatastoreServicePanelUsernameErrorLabel");

    this.alertFadeTimer = this._createAlertTimer();

    this.listener = null;

    this._init();
};


jscape.DbDatastoreServicePanel.prototype = {

    TIME_FACTOR: 60 * 1000,

    show: function() {
        this.panel.show();
    },

    hide: function() {
        this.panel.hide();
    },

    reset: function() {
        this.setUrl("");
        this.setUsername("");
        this.setPassword("");
    },

    setListener: function(listener) {
        this.listener = listener;
    },

    applyChanges: function() {
        this.listener.onApply();
    },

    getUrl: function() {
        return this.urlField.getValue();
    },

    setUrl: function(value) {
        this.urlField.setValue(value);
    },

    getUsername: function() {
        return this.usernameField.getValue();
    },

    setUsername: function(value) {
        this.usernameField.setValue(value);
    },

    getPassword: function() {
        return this.passwordField.getValue();
    },

    setPassword: function(value) {
        this.passwordField.setValue(value);
    },

    getPool: function() {
        return this.poolField.spinner("value");
    },

    setPool: function(value) {
        this.poolField.spinner("value", value);
    },

    getTtl: function() {
        return this.ttlField.spinner("value") * this.TIME_FACTOR;
    },

    setTtl: function(value) {
        this.ttlField.spinner("value", value / this.TIME_FACTOR);
    },

    showBadUrlError: function() {
        this.urlErrorLabel.show();
        this.alertFadeTimer.start();
    },

    showBadUsernameError: function() {
        this.usernameErrorLabel.show();
        this.alertFadeTimer.start();
    },

    clearErrors: function() {
        this.alertFadeTimer.cancel();
        this.urlErrorLabel.hide();
        this.usernameErrorLabel.hide();
    },

    _createAlertTimer: function() {
        var view = this;
        return new jscape.Timer(function(){
            view.clearErrors();
        }, 5000, false);
    },

    _init: function() {
        this.poolField.spinner({min: 1, max: 999});
        this.ttlField.spinner({min: 1, max: 999});

        this._initListeners();
    },

    _initListeners: function() {
        var view = this;
    }
};


jscape.FileLogServiceController = function(panel) {
    this.panel = panel;

    this.listener = null;
};

jscape.FileLogServiceController.prototype = {

    DESCRIPTOR_CLASS: "com.jscape.inet.mft.management.descriptors.FileLogServiceDescriptor",
    DEFAULT_DIRECTORY: "%installdir%/logs/%domain%",
    DEFAULT_STRATEGY: "Daily",
    DEFAULT_FILE_SIZE: 10485760,

    start: function(listener, parameters, keys) {
        this.listener = listener;

        this.panel.reset();

        this.panel.setDirectory(this.DEFAULT_DIRECTORY);
        this.panel.setStrategy(this.DEFAULT_STRATEGY);
        this.panel.setFileSize(this.DEFAULT_FILE_SIZE);

        this.panel.setListener(this);
    },

    onApply: function() {
        try {
            this._validate();
            var descriptor = this.getDescriptor();
            this.listener.onControllerCompleted(descriptor);
        } catch (e) {
            this._processInputError(e);
            this.listener.onControllerBroken();
        }
    },

    getDescriptor: function() {
        return {
            "@class": this.DESCRIPTOR_CLASS,
            directory: this.panel.getDirectory(),
            strategyName: this.panel.getStrategy(),
            fileSize: this.panel.getFileSize()
        };
    },

    setDescriptor: function(descriptor) {
        this.panel.setDirectory(descriptor.directory);
        this.panel.setStrategy(descriptor.strategyName);
        this.panel.setFileSize(descriptor.fileSize);
    },

    _validate: function() {
        this.panel.clearErrors();

        if (!this.panel.getDirectory().length) {
            throw new jscape.ServiceControllerException();
        }
    },

    _processInputError: function(error) {
        this.panel.showBadDirectoryError();
    }

};


jscape.FileLogServicePanel = function() {
    this.panel = $("#fileLogServicePanel");
    this.directoryField = new jscape.TextField($("#fileLogServicePanelDirectoryField"));
    this.directoryErrorLabel = $("#fileLogServicePanelDirectoryErrorLabel");
    this.dailyButton = new jscape.CheckBox($("#fileLogServicePanelDailyButton"));
    this.weeklyButton = new jscape.CheckBox($("#fileLogServicePanelWeeklyButton"));
    this.monthlyButton = new jscape.CheckBox($("#fileLogServicePanelMonthlyButton"));
    this.sizeButton = new jscape.CheckBox($("#fileLogServicePanelSizeButton"));
    this.maxFileSizeField = $("#fileLogServicePanelMaxFileSizeField");

    this.alertFadeTimer = this._createAlertTimer();

    this.strategyButtons = [
            this.dailyButton,
            this.weeklyButton,
            this.monthlyButton,
            this.sizeButton
    ];

    this.listener = null;

    this._init();
};

jscape.FileLogServicePanel.prototype = {

    FILE_SIZE_FACTOR: 1024 * 1024,

    show: function() {
        this.panel.show();
    },

    hide: function() {
        this.panel.hide();
    },

    reset: function() {
        this.setDirectory("");
    },

    setListener: function(listener) {
        this.listener = listener;
    },

    applyChanges: function() {
        this.listener.onApply();
    },

    getDirectory: function() {
        return this.directoryField.getValue();
    },

    setDirectory: function(value) {
        this.directoryField.setValue(value);
    },

    getStrategy: function() {
        for (var i = 0; i < this.strategyButtons.length; ++i) {
            var button = this.strategyButtons[i];
            if (button.isSelected()) {
                return button.getValue();
            }
        }

        throw new jscape.ServiceControllerException();
    },

    setStrategy: function(value) {
        for (var i = 0; i < this.strategyButtons.length; ++i) {
            var button = this.strategyButtons[i];
            button.setSelected(button.getValue() === value);
        }
    },

    getFileSize: function() {
        return (this.maxFileSizeField.spinner("value") * this.FILE_SIZE_FACTOR).toFixed(0);
    },

    setFileSize: function(value) {
        this.maxFileSizeField.spinner("value", (value / this.FILE_SIZE_FACTOR).toFixed(0));
    },

    showBadDirectoryError: function() {
        this.directoryErrorLabel.show();
        this.alertFadeTimer.start();
    },

    clearErrors: function() {
        this.alertFadeTimer.cancel();
        this.directoryErrorLabel.hide();
    },

    _createAlertTimer: function() {
        var view = this;
        return new jscape.Timer(function(){
            view.clearErrors();
        }, 5000, false);
    },

    _init: function() {
        this.maxFileSizeField.spinner({min: 1, max: 100});
        this.maxFileSizeField.parent().addClass("smallT");

        this._initListeners();
        this._initState();
    },

    _initListeners: function() {
        var view = this;

        this.dailyButton.field.click(function(){
            view._onDailyButtonClick();
        });

        this.weeklyButton.field.click(function(){
            view._onWeeklyButtonClick();
        });

        this.monthlyButton.field.click(function(){
            view._onMonthlyButtonClick();
        });

        this.sizeButton.field.click(function(){
            view._onSizeButtonClick();
        });
    },

    _initState: function() {
        this.maxFileSizeField.spinner("disable");
    },

    _onSizeButtonClick: function() {
        this.maxFileSizeField.spinner("enable");
    },

    _onDailyButtonClick: function() {
        this.maxFileSizeField.spinner("disable");
    },

    _onWeeklyButtonClick: function() {
        this.maxFileSizeField.spinner("disable");
    },

    _onMonthlyButtonClick: function() {
        this.maxFileSizeField.spinner("disable");
    }
};


jscape.DbLogServiceController = function(panel) {
    this.panel = panel;

    this.listener = null;
};

jscape.DbLogServiceController.prototype = {

    DESCRIPTOR_CLASS: "com.jscape.inet.mft.management.descriptors.DbLogServiceDescriptor",
    DEFAULT_POOL: 10,
    DEFAULT_TTL: 10 * 60 * 1000,

    start: function(listener, parameters, keys) {
        this.listener = listener;

        this.panel.reset();

        this.panel.setPool(this.DEFAULT_POOL);
        this.panel.setTtl(this.DEFAULT_TTL);

        this.panel.setListener(this);
    },

    onApply: function() {
        try {
            this._validate();
            var descriptor = this.getDescriptor();
            this.listener.onControllerCompleted(descriptor);
        } catch (e) {
            this._processInputError(e);
            this.listener.onControllerBroken();
        }
    },

    getDescriptor: function() {
        return {
            "@class": this.DESCRIPTOR_CLASS,
            url: this.panel.getUrl(),
            username: this.panel.getUsername(),
            password: this.panel.getPassword(),
            poolSize: this.panel.getPool(),
            idleConnectionTTL: this.panel.getTtl()
        };
    },

    setDescriptor: function(descriptor) {
        this.panel.setUrl(descriptor.url);
        this.panel.setUsername(descriptor.username);
        this.panel.setPassword(descriptor.password);
        this.panel.setPool(descriptor.poolSize);
        this.panel.setTtl(descriptor.idleConnectionTTL);
    },

    _validate: function() {
        this.panel.clearErrors();

        if (!this.panel.getUrl().length) {
            throw new jscape.BadUrlException();
        }

        if (!this.panel.getUsername().length) {
            throw new jscape.BadUsernameException();
        }
    },

    _processInputError: function(error) {
        if (error instanceof jscape.BadUrlException) {
            this.panel.showBadUrlError();
        } else if (error instanceof jscape.BadUsernameException) {
            this.panel.showBadUsernameError();
        }
    }

};

jscape.DbLogServicePanel = function() {
    this.panel = $("#dbLogServicePanel");
    this.urlField = new jscape.TextField($("#dbLogServicePanelUrlField"));
    this.usernameField = new jscape.TextField($("#dbLogServicePanelUsernameField"));
    this.passwordField = new jscape.TextField($("#dbLogServicePanelPasswordField"));
    this.poolField = $("#dbLogServicePanelPoolField");
    this.ttlField = $("#dbLogServicePanelTtlField");
    this.urlErrorLabel = $("#dbLogServicePanelUrlErrorLabel");
    this.usernameErrorLabel = $("#dbLogServicePanelUsernameErrorLabel");

    this.alertFadeTimer = this._createAlertTimer();

    this.listener = null;

    this._init();
};


jscape.DbLogServicePanel.prototype = {

    TIME_FACTOR: 60 * 1000,

    show: function() {
        this.panel.show();
    },

    hide: function() {
        this.panel.hide();
    },

    reset: function() {
        this.setUrl("");
        this.setUsername("");
        this.setPassword("");
    },

    setListener: function(listener) {
        this.listener = listener;
    },

    applyChanges: function() {
        this.listener.onApply();
    },

    getUrl: function() {
        return this.urlField.getValue();
    },

    setUrl: function(value) {
        this.urlField.setValue(value);
    },

    getUsername: function() {
        return this.usernameField.getValue();
    },

    setUsername: function(value) {
        this.usernameField.setValue(value);
    },

    getPassword: function() {
        return this.passwordField.getValue();
    },

    setPassword: function(value) {
        this.passwordField.setValue(value);
    },

    getPool: function() {
        return this.poolField.spinner("value");
    },

    setPool: function(value) {
        this.poolField.spinner("value", value);
    },

    getTtl: function() {
        return this.ttlField.spinner("value") * this.TIME_FACTOR;
    },

    setTtl: function(value) {
        this.ttlField.spinner("value", value / this.TIME_FACTOR);
    },

    showBadUrlError: function() {
        this.urlErrorLabel.show();
        this.alertFadeTimer.start();
    },

    showBadUsernameError: function() {
        this.usernameErrorLabel.show();
        this.alertFadeTimer.start();
    },

    clearErrors: function() {
        this.alertFadeTimer.cancel();
        this.urlErrorLabel.hide();
        this.usernameErrorLabel.hide();
    },

    _createAlertTimer: function() {
        var view = this;
        return new jscape.Timer(function(){
            view.clearErrors();
        }, 5000, false);
    },

    _init: function() {
        this.poolField.spinner({min: 1, max: 999});
        this.ttlField.spinner({min: 1, max: 999});

        this._initListeners();
    },

    _initListeners: function() {
        var view = this;
    }
};


jscape.NewDomainMessageDialog = function(dialogDivId,
                                         title,
                                         nameExistsMessageTemplate,
                                         serviceDeniedMessageTemplate,
                                         okButtonText) {
    this.dialog = new jscape.MessageDialog(dialogDivId, okButtonText);
    this.dialog.setTitle(title);

    this.nameExistsMessageTemplate = nameExistsMessageTemplate;
    this.serviceDeniedMessageTemplate = serviceDeniedMessageTemplate;
};

jscape.NewDomainMessageDialog.prototype = {

    showNameExistsError: function(domainName) {
        var message = this.nameExistsMessageTemplate.supplant({domainName: domainName});
        this.dialog.setMessage(message);
        this.dialog.show(this);
    },

    showServiceDeniedError: function(descriptor, license) {
        var message = this.serviceDeniedMessageTemplate.supplant({descriptorClass: descriptor['@class'], license: license.type});
        this.dialog.setMessage(message);
        this.dialog.show(this);
    },

    onCloseDialog: function() {
        this.dialog.hide();
    }
};


jscape.NewDomainController = function(dialog,
                                      messageDialog,
                                      domainNameController,
                                      serviceController,
                                      dataStoreServiceController,
                                      logServiceController,
                                      storeDomainOperation) {
    this.dialog = dialog;
    this.messageDialog = messageDialog;
    this.domainNameController = domainNameController;
    this.serviceController = serviceController;
    this.dataStoreServiceController = dataStoreServiceController;
    this.logServiceController = logServiceController;
    this.storeDomainOperation = storeDomainOperation;
    this.modules = [this.domainNameController, this.serviceController, this.dataStoreServiceController, this.logServiceController];
    this.listener = null;
    this.serverParameters = {};
    this.serverKeys = [];
    this.existingDomains = [];
    this.domainName = "";
    this.currentModuleIndex = 0;
    this.nextModuleIndex = null;
    this.domainDescriptor = null;
};

jscape.NewDomainController.prototype = {

    start: function(listener, serverParameters, serverKeys, existingDomains) {
        this.listener = listener;

        this.serverParameters = serverParameters;
        this.serverKeys = serverKeys;
        this.existingDomains = existingDomains;

        this._resetModules();

        this.domainName = "";
        this.currentModuleIndex = 0;
        this.nextModuleIndex = null;
        this.domainDescriptor = null;

        this._selectCurrentStep();

        this.dialog.show(this);
    },

    onPrevious: function() {
        this.nextModuleIndex = Math.max(this.currentModuleIndex - 1, 0);
        this.modules[this.currentModuleIndex].validate();
    },

    onNext: function() {
        this.nextModuleIndex = Math.min(this.currentModuleIndex + 1, this.modules.length - 1);
        this.modules[this.currentModuleIndex].validate();        
    },

    onSubmit: function() {
        this.nextModuleIndex = null;
        this.modules[this.currentModuleIndex].validate();
    },

    onCancel: function() {
        this.dialog.hide();
        this._cleanup();
        this.listener.onNewDomainControllerCancelled();
    },

    onValidated: function() {
        this._adjustDomainName();

        if (this.nextModuleIndex != null) {
            this.currentModuleIndex = this.nextModuleIndex;
            this._selectCurrentStep();
        } else {
            this._storeDomain();
        }
    },

    onValidationError: function() {
        // ok
    },

    onStoreDomainOperationCompleted: function() {
        this.dialog.hide();
        this._cleanup();
        this.listener.onNewDomainControllerCompleted(this.domainDescriptor);
    },

    onStoreDomainOperationBroken: function() {
        // ok
    },

    _cleanup: function() {
        this.serverParameters = null;
        this.serverKeys = null;
        this.existingDomains = null;    
    },

    _resetModules: function() {
        for (var i = 0; i < this.modules.length; ++i) {
            this.modules[i].reset();
        }
    },

    _adjustDomainName: function() {
        if (this.modules[this.currentModuleIndex] === this.domainNameController) {
            this.domainName = this.domainNameController.getDomainName();
        }
    },

    _selectCurrentStep: function() {
        this.modules[this.currentModuleIndex].start(this, this.serverParameters, this.serverKeys);
        this.dialog.setStep(this.currentModuleIndex + 1, this.modules.length);
    },

    _storeDomain: function() {
        try {
            this._validate();
            this._createDomainDescriptor();
            this.storeDomainOperation.start(this, this.domainDescriptor);
        } catch (e) {
            this.currentModuleIndex = 0;
            this._selectCurrentStep();
            this._processInputError(e);
        }
    },

    _validate: function() {
        this._validateName();
        this._validateServices();
    },

    _validateName: function() {
        var name = this.domainNameController.getDomainName();

        for (var i = 0; i < this.existingDomains.length; ++i) {
            if (name === this.existingDomains[i]) {
                throw new jscape.DomainExistsException();
            }
        }
    },

    _validateServices: function() {
        this._assertServiceAllowed(this.serviceController.getDescriptor());
        this._assertServiceAllowed(this.dataStoreServiceController.getDescriptor());
        this._assertServiceAllowed(this.logServiceController.getDescriptor());
    },

    _assertServiceAllowed: function(descriptor) {
        if (!jscape.descriptorAllowed(descriptor, this.serverParameters)) {
            throw new jscape.ServiceDeniedException(descriptor);
        }
    },

    _createDomainDescriptor: function() {
        this.domainDescriptor = jQuery.extend(true, {}, jscape.DOMAIN_TEMPLATE);

        this.domainDescriptor.name = this.domainNameController.getDomainName();
        this.domainDescriptor.description = this.domainNameController.getDomainDescription();
        this.domainDescriptor.serviceDescriptors.push(this.serviceController.getDescriptor());
        this.domainDescriptor.serviceDescriptors.push(this.dataStoreServiceController.getDescriptor());
        this.domainDescriptor.serviceDescriptors.push(this.logServiceController.getDescriptor());

        for (var i = 0; i < jscape.DEFAULT_SERVICES.length; ++i) {
            var descriptor = jscape.DEFAULT_SERVICES[i];
            if (jscape.descriptorAllowed(descriptor, this.serverParameters)) {
                this.domainDescriptor.serviceDescriptors.push(descriptor);
            }
        }
    },

    _processInputError: function(error) {
        if (error instanceof jscape.DomainExistsException) {
            this.messageDialog.showNameExistsError(this.domainName);
        } else if (error instanceof jscape.ServiceDeniedException) {
            this.messageDialog.showServiceDeniedError(error.descriptor, this.serverParameters.license);
        }
    }
};


jscape.NewDomainDialog = function(titleTemplate, previousButtonText, nextButtonText, okButtonText, cancelButtonText, panels) {
    this.dialog = $("#addDomainDialog");
    this.titleTemplate = titleTemplate;
    this.previousButtonText = previousButtonText;
    this.nextButtonText = nextButtonText;
    this.okButtonText = okButtonText;
    this.cancelButtonText = cancelButtonText;
    this.panels = panels;

    this.listener = null;

    this._init();
};

jscape.NewDomainDialog.prototype = {

    DEFAULT_WIDTH: 975,
    DEFAULT_HEIGHT: 700,

    /**
     * Shows dialog.
     *
     * @param listener a dialog listener
     */
    show: function(listener) {
        this.listener = listener;
        this.okButtonClicked = false;
        this.dialog.dialog("open");
    },

    /**
     * Hides dialog.
     */
    hide: function() {
        this.dialog.dialog("close");
    },

    select: function(panel) {
        for (var i = 0; i < this.panels.length; ++i) {
            this.panels[i].hide();
        }

        panel.show();
    },

    setStep: function(currentStep, totalSteps) {
        var title = this.titleTemplate.supplant({currentStep: currentStep, totalSteps: totalSteps});
        this.dialog.attr("title", title);
        this.dialog.dialog("option", "title", title);
    },

    _init: function() {
        var view = this;
        this.dialog.dialog({
            dialogClass: "addDomainDialog",
            width: view.DEFAULT_WIDTH,
            height: view.DEFAULT_HEIGHT,
            autoOpen: false,
            closeOnEscape: false,
            modal: true,
            resizable: true,
            buttons: this._getButtons()
        });
    },

    _getButtons: function() {
        var view = this;
        var buttons = {};
        buttons[this.previousButtonText] = function() { view._onPreviousButtonClick(); };
        buttons[this.nextButtonText] = function() { view._onNextButtonClick(); };
        buttons[this.okButtonText] = function() { view._onOkButtonClick(); };
        buttons[this.cancelButtonText] = function() { view._onCancelButtonClick(); };

        return buttons;
    },

    _onPreviousButtonClick: function() {
        this.listener.onPrevious();
    },

    _onNextButtonClick: function() {
        this.listener.onNext();
    },

    _onOkButtonClick: function() {
        this.listener.onSubmit();
    },

    _onCancelButtonClick: function() {
        this.listener.onCancel();
    }
};
