jscape.AuthenticationCredentials = function(username, password) {
    this.username = username;
    this.password = password;
};

jscape.BadUsernameException = function(){};


jscape.LoginController = function(view, loginOperation, mainUrl) {
    this.view = view;
    this.loginOperation = loginOperation;
    this.mainUrl = mainUrl;
};

jscape.LoginController.prototype = {

    start: function() {
        this.view.show(this);
    },

    onSubmit: function() {
        try {
            this._validateView();
            var credentials = new jscape.AuthenticationCredentials(this.view.getUsername(), this.view.getPassword());
            this.loginOperation.start(this, credentials);
        } catch (e) {
            this._processInputError(e);
        }
    },

    onLoginOperationCompleted: function() {
        this._openMainPage();
    },

    onLoginOperationBroken: function() {
        // nothing to do
    },

    _openMainPage: function() {
        window.location = this.mainUrl;
    },

    _validateView: function() {
        this.view.clearErrors();
        this._validateUsername();
    },

    _validateUsername: function() {
        if (!this.view.getUsername().length) {
            throw new jscape.BadUsernameException();
        }
    },

    _processInputError: function(error) {
        if (error instanceof jscape.BadUsernameException) {
            this.view.showBadUsernameError();
        }
    }

};


jscape.LoginPage = function() {
    this.alertFadeTimer = this._createAlertTimer();

    this.usernameErrorLabel = $("#usernameErrorLabel");

    this.usernameField = new jscape.TextField($("#usernameField"));
    this.passwordField = new jscape.TextField($("#passwordField"));
    this.submitButton = $("#submitButton");

    this.listener = null;

    this._init();
};

jscape.LoginPage.prototype = {

    show: function(listener) {
        this.listener = listener;
    },

    showBadUsernameError: function() {
        this.usernameErrorLabel.show();
        this.alertFadeTimer.start();
    },

    clearErrors: function() {
        this.alertFadeTimer.cancel();
        this.usernameErrorLabel.hide();
    },

    getUsername: function() {
        return this.usernameField.getValue().trim();
    },

    setUsername: function(value) {
        this.usernameField.setValue(value);
    },

    getPassword: function() {
        return this.passwordField.getValue();
    },

    setPassword: function(value) {
        this.passwordField.setValue(value);
    },

    _createAlertTimer: function() {
        var view = this;
        return new jscape.Timer(function(){
            view.clearErrors();
        }, 5000, false);
    },

    _init: function() {
        this.submitButton.button();

        this._initListeners();

        this.usernameField.focus();
    },

    _initListeners: function() {
        var view = this;

        this.usernameField.field.keydown(function(e){
            return view._onFieldKeydown(e);
        });

        this.passwordField.field.keydown(function(e){
            return view._onFieldKeydown(e);
        });

        this.submitButton.click(function(){
            view._onSubmitButtonClick();
        });
    },

    _onFieldKeydown: function(event) {
        if (jscape.isReturnKeyCode(event.keyCode)) {
            this._onSubmitButtonClick();
            return false;
        }

        return true;
    },

    _onSubmitButtonClick: function() {
        this.listener.onSubmit();
    }

};
