jscape.Hours = function(value) {
    this.SECONDS_PER_MINUTE = 60;
    this.MINUTES_PER_HOUR = 60;
    this.MILLISECONDS_PER_SECOND = 1000;
    this.MILLISECONDS_PER_MINUTE = this.MILLISECONDS_PER_SECOND * this.SECONDS_PER_MINUTE;
    this.MILLISECONDS_PER_HOUR = this.MILLISECONDS_PER_MINUTE * this.MINUTES_PER_HOUR;

    this.hours = value / this.MILLISECONDS_PER_HOUR | 0;
    value %= this.MILLISECONDS_PER_HOUR;
    this.minutes = value / this.MILLISECONDS_PER_MINUTE | 0;
    value %= this.MILLISECONDS_PER_MINUTE;
    this.seconds = value / this.MILLISECONDS_PER_SECOND | 0;
    this.millis = value % this.MILLISECONDS_PER_SECOND | 0;
};

jscape.ServerStatisticsUpdater = function(serverStatisticsOperation, updatePeriodMillis) {
    this.serverStatisticsOperation = serverStatisticsOperation;
    this.updatePeriodMillis = updatePeriodMillis;
    this.timer = null;
};


jscape.mbytes = function(bytes) {
    return (bytes / (1024 * 1024)).toFixed(2);
};


jscape.ServerStatisticsUpdater.prototype = {

    start: function(listener) {
        this.listener = listener;
        this._startTimer();
    },

    onServerStatisticsOperationCompleted: function(statistics) {
        this.listener.onServerStatistics(statistics);
        this._startTimer();
    },

    onServerStatisticsOperationBroken: function() {
        this._startTimer();
    },

    _startTimer: function() {
        var controller = this;
        this.timer = new jscape.Timer(function(){
            controller._startOperation();
        }, this.updatePeriodMillis, false);

        this.timer.start();
    },

    _startOperation: function() {
        this.serverStatisticsOperation.start(this);
    }
};


jscape.UnderConstructionDialog = function() {
    this.dialog = new jscape.MessageDialog("underConstructionDialog", "OK");
    this.dialog.setTitle("Under Construction");
    this.dialog.setMessage("The feature is under construction.");
};

jscape.UnderConstructionDialog.prototype = {

    show: function() {
        this.dialog.show(this);
    },

    onCloseDialog: function() {
        this.dialog.hide();
    }

};


jscape.DeleteDomainConfirmationDialog = function(dialogDivId, confirmButtonText, cancelButtonText, title, message){
    this.dialog = new jscape.ConfirmationDialog(dialogDivId, confirmButtonText, cancelButtonText);
    this.dialog.setTitle(title);
    this.message = message;

    this.listener = null;
};

jscape.DeleteDomainConfirmationDialog.prototype = {

    show: function(listener) {
        this.listener = listener;
        this.dialog.show(this);
    },

    hide: function() {
        this.dialog.hide();
    },

    setDomain: function(domainName) {
        var message = this.message.supplant({domainName: domainName});
        this.dialog.setMessage(message);
    },

    onConfirmDialog: function() {
        this.listener.onDomainDeleteConfirmed();
    },

    onCancelDialog: function() {
        this.listener.onDomainDeleteCancelled();
    }

};


jscape.MainController = function(view,
                                 serverParametersOperation,
                                 serverStatisticsOperation,
                                 serverKeysOperation,
                                 domainsOperation,
                                 startDomainsOperation,
                                 stopDomainsOperation,
                                 pauseDomainsOperation,
                                 resumeDomainsOperation,
                                 startDomainOperation,
                                 stopDomainOperation,
                                 pauseDomainOperation,
                                 resumeDomainOperation,
                                 deleteDomainOperation,
                                 logoutOperation,
                                 newDomainController,
                                 deleteDomainConfirmationDialog,                                 
                                 serverKeysUrl,
                                 serverSettingsUrl,
                                 contentsHelpUrl,
                                 restApiHelpUrl,
                                 loginUrl,
                                 editDomainUrl,
                                 statusDomainUrl,
                                 updatePeriodMillis) {
    this.view = view;
    this.serverParametersOperation = serverParametersOperation;
    this.serverStatisticsOperation = serverStatisticsOperation;
    this.serverKeysOperation = serverKeysOperation;
    this.domainsOperation = domainsOperation;
    this.startDomainsOperation = startDomainsOperation;
    this.stopDomainsOperation = stopDomainsOperation;
    this.pauseDomainsOperation = pauseDomainsOperation;
    this.resumeDomainsOperation = resumeDomainsOperation; 
    this.startDomainOperation = startDomainOperation;
    this.stopDomainOperation = stopDomainOperation;
    this.pauseDomainOperation = pauseDomainOperation;
    this.resumeDomainOperation = resumeDomainOperation;
    this.deleteDomainOperation = deleteDomainOperation;
    this.logoutOperation = logoutOperation;
    this.newDomainController = newDomainController;
    this.deleteDomainConfirmationDialog = deleteDomainConfirmationDialog;
    this.serverKeysUrl = serverKeysUrl;
    this.serverSettingsUrl = serverSettingsUrl;
    this.contentsHelpUrl = contentsHelpUrl;
    this.restApiHelpUrl = restApiHelpUrl;
    this.loginUrl = loginUrl;
    this.editDomainUrl = editDomainUrl;
    this.statusDomainUrl = statusDomainUrl;
    this.statisticsUpdater = new jscape.ServerStatisticsUpdater(this.serverStatisticsOperation, updatePeriodMillis);
    this.serverParameters = null;
    this.serverKeys = null;
    this.currentDomain = null;
};

jscape.MainController.prototype = {

    start: function() {
        this.serverParametersOperation.start(this);
    },

    onServerParametersOperationCompleted: function(parameters) {
        this.serverParameters = parameters;
        this.view.setParameters(parameters);
        this.serverStatisticsOperation.start(this);
    },

    onServerParametersOperationBroken: function() {
        this.logoutOperation.start(this);
    },

    onServerStatisticsOperationCompleted: function(statistics) {
        this.serverStatistics = statistics;
        this.view.setStatistics(statistics);
        this.view.show(this);
        this.statisticsUpdater.start(this);
    },

    onServerStatisticsOperationBroken: function() {
        this.logoutOperation.start(this);
    },

    onServerKeysOperationCompleted: function(keys) {
        this.serverKeys = keys;
        this.domainsOperation.start(this);
    },

    onServerKeysOperationBroken: function() {
        // ok
    },

    onDomainsOperationCompleted: function(domains) {
         this.newDomainController.start(this, this.serverParameters, this.serverKeys, domains);
    },

    onDomainsOperationBroken: function() {
        // ok
    },

    onStartDomainsOperationCompleted: function() {
        // ok
    },

    onStartDomainsOperationBroken: function() {
        // ok
    },

    onStopDomainsOperationCompleted: function() {
        // ok
    },

    onStopDomainsOperationBroken: function() {
        // ok
    },

    onPauseDomainsOperationCompleted: function() {
        // ok
    },

    onPauseDomainsOperationBroken: function() {
        // ok
    },

    onResumeDomainsOperationCompleted: function() {
        // ok
    },

    onResumeDomainsOperationBroken: function() {
        // ok
    },

    onLogoutOperationCompleted: function() {
        this._openLoginPage();
    },

    onLogoutOperationBroken: function() {
        this._openLoginPage();
    },

    onStartDomainOperationCompleted: function() {
        this.serverStatisticsOperation.start(this);
    },

    onStartDomainOperationBroken: function() {
        // ok
    },

    onStopDomainOperationCompleted: function() {
        this.serverStatisticsOperation.start(this);
    },

    onStopDomainOperationBroken: function() {
        // ok
    },

    onPauseDomainOperationCompleted: function() {
        this.serverStatisticsOperation.start(this);
    },

    onPauseDomainOperationBroken: function() {
        // ok
    },

    onResumeDomainOperationCompleted: function() {
        this.serverStatisticsOperation.start(this);
    },

    onResumeDomainOperationBroken: function() {
        // ok
    },

    onDeleteDomainOperationCompleted: function() {
        this.serverStatisticsOperation.start(this);
    },

    onDeleteDomainOperationBroken: function() {
        // ok
    },

    onStartDomains: function() {
        this.startDomainsOperation.start(this);
    },

    onStopDomains: function() {
        this.stopDomainsOperation.start(this);
    },

    onPauseDomains: function() {
        this.pauseDomainsOperation.start(this);
    },

    onResumeDomains: function() {
        this.resumeDomainsOperation.start(this);
    },

    onServerKeys: function() {
        // todo
        new jscape.UnderConstructionDialog().show();
    },

    onServerSettings: function() {
        // todo
        new jscape.UnderConstructionDialog().show();
    },

    onContentsHelp: function() {
        this._openTab(this.contentsHelpUrl);
    },

    onRestApiHelp: function() {
        this._openTab(this.restApiHelpUrl);
    },

    onLogout: function() {
        this.logoutOperation.start(this);
    },

    onStartDomain: function(domainName) {
        this.startDomainOperation.start(this, domainName);
    },

    onStopDomain: function(domainName) {
        this.stopDomainOperation.start(this, domainName);
    },

    onPauseDomain: function(domainName) {
        this.pauseDomainOperation.start(this, domainName);
    },

    onResumeDomain: function(domainName) {
        this.resumeDomainOperation.start(this, domainName);
    },

    onAddDomain: function() {
        this.serverKeysOperation.start(this);
    },

    onEditDomain: function(domainName) {
        // todo
//        this._openEditDomainPage(domainName);
        new jscape.UnderConstructionDialog().show();
    },

    onStatusDomain: function(domainName) {
        // todo
//        this._openStatusDomainPage(domainName);
        new jscape.UnderConstructionDialog().show();
    },

    onDeleteDomain: function(domainName) {
        this.currentDomain = domainName;
        this.deleteDomainConfirmationDialog.setDomain(domainName);
        this.deleteDomainConfirmationDialog.show(this);
    },

    onNewDomainControllerCompleted: function(descriptor) {
        this.startDomainOperation.start(this, descriptor.name);     
    },

    onNewDomainControllerCancelled: function() {
        // ok
    },

    onServerStatistics: function(statistics) {
        this.serverStatistics = statistics;
        this.view.setStatistics(statistics);
    },

    onDomainDeleteConfirmed: function() {
        this.deleteDomainConfirmationDialog.hide();
        this.deleteDomainOperation.start(this, this.currentDomain);
    },

    onDomainDeleteCancelled: function() {
        this.deleteDomainConfirmationDialog.hide();
    },

    _openTab: function(url) {
        var tab = window.open();
        tab.location = url;
        tab.focus();
    },

    _openLoginPage: function() {
        window.location = this.loginUrl;
    },

    _openEditDomainPage: function(domainName) {
        window.location = this.editDomainUrl.supplant({domainName: domainName});
    },

    _openStatusDomainPage: function(domainName) {
        window.location = this.statusDomainUrl.supplant({domainName: domainName});
    }
};


jscape.DomainRow = function(row, listener) {
    this.row = row;
    this.domainName = this.row.find("td:first").text();
    this.listener = listener;

    this._init();
};

jscape.DomainRow.prototype = {

    getDomainName: function() {
        return this.domainName;
    },

    select: function() {
        this.row.css("background-color", "#FFF1BE");
    },

    unselect: function() {
        this.row.css("background-color", "#FDFDFD");
    },

    _init: function() {
        var view = this;
        this.row.click(function(){
            view.listener.onDomainRowSelected(view);
        });
    }
};


jscape.MainPage = function() {
    this.newDomainMenuItem = $("#newDomainMenuItem");
    this.startDomainsMenuItem = $("#startDomainsMenuItem");
    this.stopDomainsMenuItem = $("#stopDomainsMenuItem");
    this.pauseDomainsMenuItem = $("#pauseDomainsMenuItem");
    this.resumeDomainsMenuItem = $("#resumeDomainsMenuItem");
    this.serverKeysMenuItem = $("#serverKeysMenuItem");
    this.serverSettingsMenuItem = $("#serverSettingsMenuItem");
    this.contentsMenuItem = $("#contentsMenuItem");
    this.restApiHelpMenuItem = $("#restApiHelpMenuItem");
    this.logoutMenuItem = $("#logoutMenuItem");

    this.serverTab = $("#serverTab");
    this.domainsTab = $("#domainsTab");
    this.networkTab = $("#networkTab");

    this.serverStatisticsPanel = $("#serverStatisticsPanel");
    this.serverDomainsPanel = $("#serverDomainsPanel");
    this.serverInterfacesPanel = $("#serverInterfacesPanel");

    this.serverVersionValueLabel = $("#serverVersionValueLabel");
    this.licenseTypeValueLabel = $("#licenseTypeValueLabel");
    this.licenseOwnerValueLabel = $("#licenseOwnerValueLabel");
    this.licenseHostValueLabel = $("#licenseHostValueLabel");
    this.licenseExpirationValueLabel = $("#licenseExpirationValueLabel");
    this.domainsValueLabel = $("#domainsValueLabel");
    this.uptimeValueLabel = $("#uptimeValueLabel");
    this.managementConnectionsValueLabel = $("#managementConnectionsValueLabel");
    this.connectionsValueLabel = $("#connectionsValueLabel");
    this.memoryValueLabel = $("#memoryValueLabel");
    this.threadsValueLabel = $("#threadsValueLabel");

    this.domainsTable = $("#domainsTable");
    this.domainsRows = $("#domainsRows");
    this.serverInterfacesTable = $("#serverInterfacesTable");

    this.startDomainButton = $("#startDomainButton");
    this.stopDomainButton = $("#stopDomainButton");
    this.pauseDomainButton = $("#pauseDomainButton");
    this.resumeDomainButton = $("#resumeDomainButton");
    this.addDomainButton = $("#addDomainButton");
    this.editDomainButton = $("#editDomainButton");
    this.domainStatusButton = $("#domainStatusButton");
    this.deleteDomainButton = $("#deleteDomainButton");

    this.listener = null;
    this.domains = [];
    this.domainRows = [];
    this.selectedDomain = null;

    this._init();
};

jscape.MainPage.prototype = {

    DOMAIN_RUNNING: 0,
    DOMAIN_PAUSED: 1,
    DOMAIN_STOPPED: 2,
    DOMAIN_ROW_TEMPLATE: "<tr><td>{domainName}</td><td align='center'>{state}</td><td align='center'>{currentSessions}</td><td align='center'>{currentTransfers}</td><td align='center'>{totalSessions}</td><td align='center'>{bytesUploaded}</td><td align='center'>{filesUploaded}</td><td align='center'>{bytesDownloaded}</td><td align='center'>{filesDownloaded}</td><td align='center'>{uploadsQuota}</td><td align='center'>{downloadsQuota}</td><td align='center'>{transfersQuota}</td><td align='center'>{startDate}</td><td align='center'>{stopDate}</td></tr>",
    NETWORK_INTERFACE_ROW_TEMPLATE: "<tr><td>{name}</td><td>{displayName}</td><td>{addresses}</td></tr>",

    show: function(listener) {
        this.listener = listener;
    },

    setParameters: function(parameters) {
        this.serverVersionValueLabel.text(parameters.applicationVersion);
        this.licenseTypeValueLabel.text(parameters.license.type);
        this.licenseOwnerValueLabel.text(parameters.license.owner);
        this.licenseHostValueLabel.text(parameters.license.host ? parameters.license.host : "non-binding");
        this.licenseExpirationValueLabel.text(parameters.license.expirationDate ? $.datepicker.formatDate("mm-dd-yy", new Date(parameters.license.expirationDate)) : "non-expiring");

        this._setNetworkInterfaces(parameters.interfaces);
    },

    setStatistics: function(statistics) {
        this.domainsValueLabel.text(statistics.domainStatistics[1].length);
        this.uptimeValueLabel.text("{hours} hrs. {minutes} min. {seconds} sec.".supplant(new jscape.Hours(statistics.uptime)));
        this.managementConnectionsValueLabel.text(statistics.managementConnections);
        this.connectionsValueLabel.text(this._countCurrentConnections(statistics.domainStatistics[1]));
        this.memoryValueLabel.text("{usedMemory} MB of {maxMemory} MB".supplant({usedMemory: jscape.mbytes(statistics.usedMemory), maxMemory: jscape.mbytes(statistics.maxMemory)}));
        this.threadsValueLabel.text(statistics.threads);

        this._setDomains(statistics.domainStatistics[1]);
    },

    onDomainRowSelected: function(domainRow) {
        for (var i = 0; i < this.domainRows.length; ++i) {
            this.domainRows[i].unselect();
        }

        domainRow.select();
        this.selectedDomain = domainRow.getDomainName();
        this._adjustButtonsState();
    },

    _init: function() {
        this.domainsTable.tablesorter();

        this._initListeners();
        this._initState();
    },

    _initListeners: function() {
        var view = this;

        this.newDomainMenuItem.click(function(){
            view._onNewDomainMenuItemClick();
            return false;            
        });

        this.startDomainsMenuItem.click(function(){
            view._onStartDomainsMenuItemClick();
            return false;
        });

        this.stopDomainsMenuItem.click(function(){
            view._onStopDomainsMenuItemClick();
            return false;
        });

        this.pauseDomainsMenuItem.click(function(){
            view._onPauseDomainsMenuItemClick();
            return false;
        });

        this.resumeDomainsMenuItem.click(function(){
            view._onResumeDomainsMenuItemClick();
            return false;
        });

        this.serverKeysMenuItem.click(function(){
            view._onServerKeysMenuItemClick();
            return false;
        });

        this.serverSettingsMenuItem.click(function(){
            view._onServerSettingsMenuItemClick();
            return false;
        });

        this.contentsMenuItem.click(function(){
            view._onContentsMenuItemClick();
            return false;
        });

        this.restApiHelpMenuItem.click(function(){
            view._onRestApiHelpMenuItemClick();
            return false;
        });

        this.logoutMenuItem.click(function(){
            view._onLogoutMenuItemClick();
            return false;
        });

        this.serverTab.click(function(){
            view._onServerTabClick();
            return false;
        });

        this.domainsTab.click(function(){
            view._onDomainsTabClick();
            return false;
        });

        this.networkTab.click(function(){
            view._onNetworkTabClick();
            return false;
        });

        this.startDomainButton.click(function(){
            view._onStartDomainButtonClick();
            return false;
        });

        this.stopDomainButton.click(function(){
            view._onStopDomainButtonClick();
            return false;
        });

        this.pauseDomainButton.click(function(){
            view._onPauseDomainButtonClick();
            return false;
        });

        this.resumeDomainButton.click(function(){
            view._onResumeDomainButtonClick();
            return false;
        });

        this.addDomainButton.click(function(){
            view._onAddDomainButtonClick();
            return false;
        });

        this.editDomainButton.click(function(){
            view._onEditDomainButtonClick();
            return false;
        });

        this.domainStatusButton.click(function(){
            view._onDomainStatusButtonClick();
            return false;
        });

        this.deleteDomainButton.click(function(){
            view._onDeleteDomainButtonClick();
            return false;
        });
    },

    _initState: function() {
        this._initButtonsState();
    },

    _initButtonsState: function() {
        this.startDomainButton.addClass("disabled");
        this.stopDomainButton.addClass("disabled");
        this.pauseDomainButton.addClass("disabled");
        this.resumeDomainButton.addClass("disabled");
        this.editDomainButton.addClass("disabled");
        this.domainStatusButton.addClass("disabled");
        this.deleteDomainButton.addClass("disabled");
    },

    _adjustButtonsState: function() {
        var domain = this._currentDomain();
        var state = this._stateFor(domain);

        if (state === this.DOMAIN_STOPPED) {
            this.startDomainButton.removeClass("disabled");
        } else {
            this.startDomainButton.addClass("disabled");
        }

        if (state !== this.DOMAIN_STOPPED) {
            this.stopDomainButton.removeClass("disabled");
        } else {
            this.stopDomainButton.addClass("disabled");
        }

        if (state === this.DOMAIN_RUNNING) {
            this.pauseDomainButton.removeClass("disabled");
        } else {
            this.pauseDomainButton.addClass("disabled");
        }

        if (state === this.DOMAIN_PAUSED) {
            this.resumeDomainButton.removeClass("disabled");
        } else {
            this.resumeDomainButton.addClass("disabled");
        }

        this.editDomainButton.removeClass("disabled");
        this.domainStatusButton.removeClass("disabled");
        this.deleteDomainButton.removeClass("disabled");
    },

    _onNewDomainMenuItemClick: function() {
        this.listener.onAddDomain();
    },

    _onStartDomainsMenuItemClick: function() {
        this.listener.onStartDomains();
    },

    _onStopDomainsMenuItemClick: function() {
        this.listener.onStopDomains();
    },

    _onPauseDomainsMenuItemClick: function() {
        this.listener.onPauseDomains();
    },

    _onResumeDomainsMenuItemClick: function() {
        this.listener.onResumeDomains();
    },

    _onServerKeysMenuItemClick: function() {
        this.listener.onServerKeys();
    },

    _onServerSettingsMenuItemClick: function() {
        this.listener.onServerSettings();
    },

    _onContentsMenuItemClick: function() {
        this.listener.onContentsHelp();
    },

    _onRestApiHelpMenuItemClick: function() {
        this.listener.onRestApiHelp();
    },     

    _onLogoutMenuItemClick: function() {
        this.listener.onLogout();
    },

    _onServerTabClick: function() {
        this.serverTab.addClass("activeThis");
        this.serverStatisticsPanel.show();

        this.domainsTab.removeClass("activeThis");
        this.serverDomainsPanel.hide();

        this.networkTab.removeClass("activeThis");
        this.serverInterfacesPanel.hide();
    },

    _onDomainsTabClick: function() {
        this.serverTab.removeClass("activeThis");
        this.serverStatisticsPanel.hide();

        this.serverDomainsPanel.show();
        this.domainsTab.addClass("activeThis");

        this.networkTab.removeClass("activeThis");
        this.serverInterfacesPanel.hide();
    },

    _onNetworkTabClick: function() {
        this.serverTab.removeClass("activeThis");
        this.serverStatisticsPanel.hide();

        this.serverInterfacesPanel.show();
        this.serverDomainsPanel.hide();

        this.networkTab.addClass("activeThis");
        this.domainsTab.removeClass("activeThis");
    },

    _onStartDomainButtonClick: function() {
        this.listener.onStartDomain(this.selectedDomain);
    },

    _onStopDomainButtonClick: function() {
        this.listener.onStopDomain(this.selectedDomain);
    },

    _onPauseDomainButtonClick: function() {
        this.listener.onPauseDomain(this.selectedDomain);
    },

    _onResumeDomainButtonClick: function() {
        this.listener.onResumeDomain(this.selectedDomain);
    },

    _onAddDomainButtonClick: function() {
        this.listener.onAddDomain();
    },

    _onEditDomainButtonClick: function() {
        this.listener.onEditDomain(this.selectedDomain);
    },

    _onDomainStatusButtonClick: function() {
        this.listener.onStatusDomain(this.selectedDomain);
    },

    _onDeleteDomainButtonClick: function() {
        this.listener.onDeleteDomain(this.selectedDomain);
    },

    _countCurrentConnections: function(domainStatistics) {
        var connections = 0;
        for (var i = 0; i < domainStatistics.length; ++i) {
            connections += domainStatistics[i].currentSessions;
        }

        return connections;
    },

    _setDomains: function(domains) {
        this.domains = domains;
        this.domainsRows.text("");
        this.domainRows = [];
        this._initButtonsState();

        for (var i = 0; i < domains.length; ++i) {
            var domain = domains[i];
            var rowData = {
                domainName: domain.domainName,
                state: this._stateMessageFor(this._stateFor(domain)),
                currentSessions: domain.currentSessions,
                currentTransfers: domain.currentTransfers,
                totalSessions: domain.totalSessions,
                bytesUploaded: jscape.mbytes(domain.bytesUploaded) + " MB",
                filesUploaded: domain.filesUploaded,
                bytesDownloaded: jscape.mbytes(domain.bytesDownloaded) + " MB",
                filesDownloaded: domain.filesDownloaded,
                uploadsQuota: domain.uploadsQuota ? jscape.mbytes(domain.bytesUploaded) + " MB out of " + jscape.mbytes(domain.uploadsQuota) + " MB" : "",
                downloadsQuota: domain.downloadsQuota ? jscape.mbytes(domain.bytesDownloaded) + " MB out of " + jscape.mbytes(domain.downloadsQuota) + " MB" : "",
                transfersQuota: domain.transfersQuota ? jscape.mbytes(domain.bytesUploaded + domain.bytesDownloaded) + " MB out of " + jscape.mbytes(domain.transfersQuota) + " MB" : "",
                startDate: domain.startDate ? $.datepicker.formatDate("mm-dd-yy", new Date(domain.startDate)) : "",
                stopDate: domain.stopDate ? $.datepicker.formatDate("mm-dd-yy", new Date(domain.stopDate)) : ""
            };
            var row = this.DOMAIN_ROW_TEMPLATE.supplant(rowData);
            this.domainsRows.append(row);

            var domainRow = new jscape.DomainRow(this.domainsRows.find("tr:last"), this);
            this.domainRows.push(domainRow);
            
            if (domain.domainName === this.selectedDomain) {
                domainRow.select();
                this._adjustButtonsState();
            }
        }

        this.domainsTable.trigger("update", [true]);
    },

    _setNetworkInterfaces: function(interfaces) {
        this.serverInterfacesTable.text("");
        
        for (var i = 0; i < interfaces.length; ++i) {
            var networkInterface = interfaces[i];
            var rowData = { name: networkInterface.name, displayName: networkInterface.displayName, addresses: networkInterface.addresses.join() };
            var row = this.NETWORK_INTERFACE_ROW_TEMPLATE.supplant(rowData);
            this.serverInterfacesTable.append(row);
        }
    },

    _currentDomain: function() {
        for (var i = 0; i < this.domains.length; ++i) {
            var domain = this.domains[i];
            if (domain.domainName === this.selectedDomain) {
                return domain;
            }
        }

        return null;
    },

    _stateFor: function(domain) {
        return domain.running && domain.paused
               ? this.DOMAIN_PAUSED
               : domain.running ? this.DOMAIN_RUNNING : this.DOMAIN_STOPPED;
    },

    _stateMessageFor: function(state) {
        return state === this.DOMAIN_RUNNING ? "running" : state === this.DOMAIN_PAUSED ? "paused" : "stopped"
    }

};

