jscape.LoginOperation = function(url, timeoutMillis, progressDialog, errorMessageDialog) {
    this.url = url;
    this.base = new jscape.RestOperation("POST", "application/json", "text", timeoutMillis, progressDialog, errorMessageDialog);
    this.listener = null;
};

jscape.LoginOperation.prototype = {

    start: function(listener, credentials) {
        this.listener = listener;
        var data = this._formatCredentials(credentials);
        this.base.start(this, this.url, data);
    },

    onCompleted: function() {
        this.listener.onLoginOperationCompleted();
    },

    onBroken: function() {
        this.listener.onLoginOperationBroken();
    },

    _formatCredentials: function(credentials) {
        return JSON.stringify(credentials);
    }
};


jscape.LogoutOperation = function(url, timeoutMillis, progressDialog, errorMessageDialog) {
    this.url = url;
    this.base = new jscape.RestOperation("POST", "application/json", "text", timeoutMillis, progressDialog, errorMessageDialog);
    this.listener = null;
};

jscape.LogoutOperation.prototype = {

    start: function(listener) {
        this.listener = listener;
        this.base.start(this, this.url, "");
    },

    onCompleted: function() {
        this.listener.onLogoutOperationCompleted();
    },

    onBroken: function() {
        this.listener.onLogoutOperationBroken();
    }
};


jscape.ServerParametersOperation = function(url, timeoutMillis) {
    this.url = url;
    this.base = new jscape.RestOperation("GET", "application/json", "text", timeoutMillis, new jscape.NullDialog(), new jscape.NullDialog());
    this.listener = null;
};

jscape.ServerParametersOperation.prototype = {

    start: function(listener) {
        this.listener = listener;
        this.base.start(this, this.url, "");
    },

    onCompleted: function(operation, data) {
        var parameters = $.parseJSON(data);
        this.listener.onServerParametersOperationCompleted(parameters);
    },

    onBroken: function(operation) {
        this.listener.onServerParametersOperationBroken();
    }
};


jscape.ServerStatisticsOperation = function(url, timeoutMillis) {
    this.url = url;
    this.base = new jscape.RestOperation("GET", "application/json", "text", timeoutMillis, new jscape.NullDialog(), new jscape.NullDialog());
    this.listener = null;
};

jscape.ServerStatisticsOperation.prototype = {

    start: function(listener) {
        this.listener = listener;
        this.base.start(this, this.url, "");
    },

    onCompleted: function(operation, data) {
        var statistics = $.parseJSON(data);
        this.listener.onServerStatisticsOperationCompleted(statistics);
    },

    onBroken: function(operation) {
        this.listener.onServerStatisticsOperationBroken();
    }
};


jscape.ServerKeysOperation = function(url, timeoutMillis) {
    this.url = url;
    this.base = new jscape.RestOperation("GET", "application/json", "text", timeoutMillis, new jscape.NullDialog(), new jscape.NullDialog());
    this.listener = null;
};

jscape.ServerKeysOperation.prototype = {

    start: function(listener) {
        this.listener = listener;
        this.base.start(this, this.url, "");
    },

    onCompleted: function(operation, data) {
        var keys = $.parseJSON(data);
        this.listener.onServerKeysOperationCompleted(keys);
    },

    onBroken: function(operation) {
        this.listener.onServerKeysOperationBroken();
    }
};


jscape.DomainsOperation = function(url, timeoutMillis) {
    this.url = url;
    this.base = new jscape.RestOperation("GET", "application/json", "text", timeoutMillis, new jscape.NullDialog(), new jscape.NullDialog());
    this.listener = null;
};

jscape.DomainsOperation.prototype = {

    start: function(listener) {
        this.listener = listener;
        this.base.start(this, this.url, "");
    },

    onCompleted: function(operation, data) {
        var domains = $.parseJSON(data);
        this.listener.onDomainsOperationCompleted(domains);
    },

    onBroken: function(operation) {
        this.listener.onDomainsOperationBroken();
    }
};


jscape.StartDomainsOperation = function(domainsOperation, startDomainOperation) {
    this.domainsOperation = domainsOperation;
    this.startDomainOperation = startDomainOperation;
    this.listener = null;
    this.domains = [];
    this.currentDomain = 0;
};

jscape.StartDomainsOperation.prototype = {

    start: function(listener) {
        this.listener = listener;
        this.domainsOperation.start(this);
    },

    onDomainsOperationCompleted: function(domains) {
        this.domains = domains || [];
        this.currentDomain = 0;
        this._processNextDomain();
    },

    onDomainsOperationBroken: function() {
        this.listener.onStartDomainsOperationBroken();
    },

    onStartDomainOperationCompleted: function() {
        this._processNextDomain();
    },

    onStartDomainOperationBroken: function() {
        this.listener.onStartDomainsOperationBroken();
    },

    _processNextDomain: function() {
        if (this.currentDomain < this.domains.length) {
            this.startDomainOperation.start(this, this.domains[this.currentDomain++]);
        } else {
            this.listener.onStartDomainsOperationCompleted();
        }
    }
};


jscape.StopDomainsOperation = function(domainsOperation, stopDomainOperation) {
    this.domainsOperation = domainsOperation;
    this.stopDomainOperation = stopDomainOperation;
    this.listener = null;
    this.domains = [];
    this.currentDomain = 0;
};

jscape.StopDomainsOperation.prototype = {

    start: function(listener) {
        this.listener = listener;
        this.domainsOperation.start(this);
    },

    onDomainsOperationCompleted: function(domains) {
        this.domains = domains || [];
        this.currentDomain = 0;
        this._processNextDomain();
    },

    onDomainsOperationBroken: function() {
        this.listener.onStopDomainsOperationBroken();
    },

    onStopDomainOperationCompleted: function() {
        this._processNextDomain();
    },

    onStopDomainOperationBroken: function() {
        this.listener.onStopDomainsOperationBroken();
    },

    _processNextDomain: function() {
        if (this.currentDomain < this.domains.length) {
            this.stopDomainOperation.start(this, this.domains[this.currentDomain++]);
        } else {
            this.listener.onStopDomainsOperationCompleted();
        }
    }
};


jscape.PauseDomainsOperation = function(domainsOperation, pauseDomainOperation) {
    this.domainsOperation = domainsOperation;
    this.pauseDomainOperation = pauseDomainOperation;
    this.listener = null;
    this.domains = [];
    this.currentDomain = 0;
};

jscape.PauseDomainsOperation.prototype = {

    start: function(listener) {
        this.listener = listener;
        this.domainsOperation.start(this);
    },

    onDomainsOperationCompleted: function(domains) {
        this.domains = domains || [];
        this.currentDomain = 0;
        this._processNextDomain();
    },

    onDomainsOperationBroken: function() {
        this.listener.onPauseDomainsOperationBroken();
    },

    onPauseDomainOperationCompleted: function() {
        this._processNextDomain();
    },

    onPauseDomainOperationBroken: function() {
        this.listener.onPauseDomainsOperationBroken();
    },

    _processNextDomain: function() {
        if (this.currentDomain < this.domains.length) {
            this.pauseDomainOperation.start(this, this.domains[this.currentDomain++]);
        } else {
            this.listener.onPauseDomainsOperationCompleted();
        }
    }
};


jscape.ResumeDomainsOperation = function(domainsOperation, resumeDomainOperation) {
    this.domainsOperation = domainsOperation;
    this.resumeDomainOperation = resumeDomainOperation;
    this.listener = null;
    this.domains = [];
    this.currentDomain = 0;
};

jscape.ResumeDomainsOperation.prototype = {

    start: function(listener) {
        this.listener = listener;
        this.domainsOperation.start(this);
    },

    onDomainsOperationCompleted: function(domains) {
        this.domains = domains || [];
        this.currentDomain = 0;
        this._processNextDomain();
    },

    onDomainsOperationBroken: function() {
        this.listener.onResumeDomainsOperationBroken();
    },

    onResumeDomainOperationCompleted: function() {
        this._processNextDomain();
    },

    onResumeDomainOperationBroken: function() {
        this.listener.onResumeDomainsOperationBroken();
    },

    _processNextDomain: function() {
        if (this.currentDomain < this.domains.length) {
            this.resumeDomainOperation.start(this, this.domains[this.currentDomain]);
        } else {
            this.listener.onResumeDomainsOperationCompleted();
        }
    }
};


jscape.StartDomainOperation = function(url, timeoutMillis, progressDialog, errorMessageDialog) {
    this.url = url;
    this.base = new jscape.RestOperation("POST", "application/json", "text", timeoutMillis, progressDialog, errorMessageDialog);
    this.listener = null;
};

jscape.StartDomainOperation.prototype = {

    start: function(listener, domainName) {
        this.listener = listener;
        var url = this.url.supplant({domainName: domainName});
        this.base.start(this, url, "");
    },

    onCompleted: function(operation) {
        this.listener.onStartDomainOperationCompleted();
    },

    onBroken: function(operation) {
        this.listener.onStartDomainOperationBroken();
    }
};


jscape.StopDomainOperation = function(url, timeoutMillis, progressDialog, errorMessageDialog) {
    this.url = url;
    this.base = new jscape.RestOperation("POST", "application/json", "text", timeoutMillis, progressDialog, errorMessageDialog);
    this.listener = null;
};

jscape.StopDomainOperation.prototype = {

    start: function(listener, domainName) {
        this.listener = listener;
        var url = this.url.supplant({domainName: domainName});
        this.base.start(this, url, "");
    },

    onCompleted: function(operation) {
        this.listener.onStopDomainOperationCompleted();
    },

    onBroken: function(operation) {
        this.listener.onStopDomainOperationBroken();
    }
};


jscape.PauseDomainOperation = function(url, timeoutMillis, progressDialog, errorMessageDialog) {
    this.url = url;
    this.base = new jscape.RestOperation("POST", "application/json", "text", timeoutMillis, progressDialog, errorMessageDialog);
    this.listener = null;
};

jscape.PauseDomainOperation.prototype = {

    start: function(listener, domainName) {
        this.listener = listener;
        var url = this.url.supplant({domainName: domainName});
        this.base.start(this, url, "");
    },

    onCompleted: function(operation) {
        this.listener.onPauseDomainOperationCompleted();
    },

    onBroken: function(operation) {
        this.listener.onPauseDomainOperationBroken();
    }
};


jscape.ResumeDomainOperation = function(url, timeoutMillis, progressDialog, errorMessageDialog) {
    this.url = url;
    this.base = new jscape.RestOperation("POST", "application/json", "text", timeoutMillis, progressDialog, errorMessageDialog);
    this.listener = null;
};

jscape.ResumeDomainOperation.prototype = {

    start: function(listener, domainName) {
        this.listener = listener;
        var url = this.url.supplant({domainName: domainName});
        this.base.start(this, url, "");
    },

    onCompleted: function(operation) {
        this.listener.onResumeDomainOperationCompleted();
    },

    onBroken: function(operation) {
        this.listener.onResumeDomainOperationBroken();
    }
};


jscape.StoreDomainOperation = function(url, timeoutMillis, progressDialog, errorMessageDialog) {
    this.url = url;
    this.base = new jscape.RestOperation("PUT", "application/json", "text", timeoutMillis, progressDialog, errorMessageDialog);
    this.listener = null;
};

jscape.StoreDomainOperation.prototype = {

    start: function(listener, domain) {
        this.listener = listener;
        var data = this._formatDomain(domain);
        this.base.start(this, this.url, data);
    },

    onCompleted: function(operation) {
        this.listener.onStoreDomainOperationCompleted();
    },

    onBroken: function(operation) {
        this.listener.onStoreDomainOperationBroken();
    },

    _formatDomain: function(domain) {
        return JSON.stringify(domain);
    }
};


jscape.DeleteDomainOperation = function(url, timeoutMillis, progressDialog, errorMessageDialog) {
    this.url = url;
    this.base = new jscape.RestOperation("DELETE", "application/json", "text", timeoutMillis, progressDialog, errorMessageDialog);
    this.listener = null;
};

jscape.DeleteDomainOperation.prototype = {

    start: function(listener, domainName) {
        this.listener = listener;
        var url = this.url.supplant({domainName: domainName});
        this.base.start(this, url, "");
    },

    onCompleted: function(operation) {
        this.listener.onDeleteDomainOperationCompleted();
    },

    onBroken: function(operation) {
        this.listener.onDeleteDomainOperationBroken();
    }
};



